# -*- python -*-
# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2011-2016  Serge Noiraud
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Geography for two persons
"""
# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------
import operator
from math import hypot
from html import escape

# -------------------------------------------------------------------------
#
# set up logging
#
# -------------------------------------------------------------------------
import logging

# -------------------------------------------------------------------------
#
# Gramps Modules
#
# -------------------------------------------------------------------------
from gi.repository import Gtk
from gramps.gen.const import GRAMPS_LOCALE as glocale
from gramps.gen.lib import EventRoleType, EventType
from gramps.gen.config import config
from gramps.gen.datehandler import displayer, get_date
from gramps.gen.display.name import displayer as _nd
from gramps.gen.display.place import displayer as _pd
from gramps.gen.utils.place import conv_lat_lon
from gramps.gui.views.bookmarks import PersonBookmarks
from gramps.plugins.lib.maps import constants
from gramps.plugins.lib.maps.geography import GeoGraphyView
from gramps.gui.selectors import SelectorFactory
from gramps.gen.utils.db import get_birth_or_fallback, get_death_or_fallback

# -------------------------------------------------------------------------
#
# Constants
#
# -------------------------------------------------------------------------
_LOG = logging.getLogger("GeoGraphy.geoclose")
_ = glocale.translation.gettext

_UI_DEF = [
    """
      <placeholder id="CommonGo">
      <section>
        <item>
          <attribute name="action">win.Back</attribute>
          <attribute name="label" translatable="yes">_Back</attribute>
        </item>
        <item>
          <attribute name="action">win.Forward</attribute>
          <attribute name="label" translatable="yes">_Forward</attribute>
        </item>
      </section>
      <section>
        <item>
          <attribute name="action">win.HomePerson</attribute>
          <attribute name="label" translatable="yes">_Home</attribute>
        </item>
      </section>
      </placeholder>
    """,
    """
      <section id='CommonEdit' groups='RW'>
        <item>
          <attribute name="action">win.PrintView</attribute>
          <attribute name="label" translatable="yes">Print...</attribute>
        </item>
      </section>
    """,
    """
      <section id="AddEditBook">
        <item>
          <attribute name="action">win.AddBook</attribute>
          <attribute name="label" translatable="yes">_Add Bookmark</attribute>
        </item>
        <item>
          <attribute name="action">win.EditBook</attribute>
          <attribute name="label" translatable="no">%s...</attribute>
        </item>
      </section>
    """
    % _("Organize Bookmarks"),  # Following are the Toolbar items
    """
    <placeholder id='CommonNavigation'>
    <child groups='RO'>
      <object class="GtkToolButton">
        <property name="icon-name">go-previous</property>
        <property name="action-name">win.Back</property>
        <property name="tooltip_text" translatable="yes">"""
    """Go to the previous object in the history</property>
        <property name="label" translatable="yes">_Back</property>
        <property name="use-underline">True</property>
      </object>
      <packing>
        <property name="homogeneous">False</property>
      </packing>
    </child>
    <child groups='RO'>
      <object class="GtkToolButton">
        <property name="icon-name">go-next</property>
        <property name="action-name">win.Forward</property>
        <property name="tooltip_text" translatable="yes">"""
    """Go to the next object in the history</property>
        <property name="label" translatable="yes">_Forward</property>
        <property name="use-underline">True</property>
      </object>
      <packing>
        <property name="homogeneous">False</property>
      </packing>
    </child>
    <child groups='RO'>
      <object class="GtkToolButton">
        <property name="icon-name">go-home</property>
        <property name="action-name">win.HomePerson</property>
        <property name="tooltip_text" translatable="yes">"""
    """Go to the home person</property>
        <property name="label" translatable="yes">_Home</property>
        <property name="use-underline">True</property>
      </object>
      <packing>
        <property name="homogeneous">False</property>
      </packing>
    </child>
    <child groups='RO'>
      <object class="GtkToolButton">
        <property name="icon-name">gramps-person</property>
        <property name="action-name">win.RefPerson</property>
        <property name="tooltip_text" translatable="yes">"""
    """Select the person which is the reference for life ways</property>
        <property name="label" translatable="yes">reference _Person</property>
        <property name="use-underline">True</property>
      </object>
      <packing>
        <property name="homogeneous">False</property>
      </packing>
    </child>
    </placeholder>
    """,
    """
    <placeholder id='BarCommonEdit'>
    <child groups='RO'>
      <object class="GtkToolButton">
        <property name="icon-name">document-print</property>
        <property name="action-name">win.PrintView</property>
        <property name="tooltip_text" translatable="yes">"""
    """Print or save the Map</property>
        <property name="label" translatable="yes">Print...</property>
        <property name="use-underline">True</property>
      </object>
      <packing>
        <property name="homogeneous">False</property>
      </packing>
    </child>
    </placeholder>
    """,
]

# pylint: disable=no-member
# pylint: disable=unused-argument


# -------------------------------------------------------------------------
#
# GeoView
#
# -------------------------------------------------------------------------
class GeoClose(GeoGraphyView):
    """
    The view used to render person map.
    """

    CONFIGSETTINGS = (
        ("geography.path", constants.GEOGRAPHY_PATH),
        ("geography.zoom", 10),
        ("geography.zoom_when_center", 12),
        ("geography.show_cross", True),
        ("geography.lock", False),
        ("geography.center-lat", 0.0),
        ("geography.center-lon", 0.0),
        ("geography.use-keypad", True),
        ("geography.personal-map", ""),
        ("geography.map_service", constants.OPENSTREETMAP),
        ("geography.max_places", 5000),
        # specific to geoclose :
        ("geography.color1", "blue"),
        ("geography.color2", "green"),
        ("geography.maximum_meeting_zone", 5),
    )

    def __init__(self, pdata, dbstate, uistate, nav_group=0):
        GeoGraphyView.__init__(
            self,
            _("Have they been able to meet?"),
            pdata,
            dbstate,
            uistate,
            PersonBookmarks,
            nav_group,
        )
        self.dbstate = dbstate
        self.uistate = uistate
        self.place_list = []
        self.all_place_list = []
        self.place_without_coordinates = []
        self.minlat = self.maxlat = self.minlon = self.maxlon = 0.0
        self.minyear = 9999
        self.maxyear = 0
        self.refperson = None
        self.refperson_bookmark = None
        self.nbplaces = 0
        self.nbmarkers = 0
        self.sort = []
        self.tracks = []
        self.additional_uis.append(self.additional_ui())
        self.ref_person = None
        self.skip_list = []
        self.track = []
        self.place_list_active = []
        self.place_list_ref = []
        self.cal = config.get("preferences.calendar-format-report")
        self.no_show_places_in_status_bar = False
        self.add_item = None
        self.newmenu = None
        self.config_meeting_slider = None
        self.dbstate.connect("database-changed", self.reset_change_db)

    def reset_change_db(self, dummy_dbase):
        """
        Used to reset the family reference
        """
        self.refperson = None

    def get_title(self):
        """
        Used to set the titlebar in the configuration window.
        """
        return _("GeoClose")

    def get_stock(self):
        """
        Returns the name of the stock icon to use for the display.
        This assumes that this icon has already been registered
        as a stock icon.
        """
        return "gramps-relation"

    def get_viewtype_stock(self):
        """Type of view in category"""
        return "geo-show-family"

    def additional_ui(self):
        """
        Specifies the UIManager XML code that defines the menus and buttons
        associated with the interface.
        """
        return _UI_DEF

    def navigation_type(self):
        """
        Indicates the navigation type. Navigation type can be the string
        name of any of the primary objects.
        """
        return "Person"

    def goto_handle(self, handle=None):
        """
        Rebuild the tree with the given person handle as the root.
        """
        if self.osm is None:
            return
        self.place_list_active = []
        self.place_list_ref = []
        self.all_place_list = []
        self.sort = []
        self.places_found = []
        self.nbmarkers = 0
        self.nbplaces = 0
        self.place_without_coordinates = []
        self.remove_all_gps()
        self.remove_all_markers()
        self.lifeway_layer.clear_ways()
        self.message_layer.clear_messages()
        self.message_layer.set_font_attributes(None, None, None)
        active = self.get_active()
        if active:
            indiv1 = self.dbstate.db.get_person_from_handle(active)
            color = self._config.get("geography.color2")
            self._createmap(indiv1, color, self.place_list_active, False)
        if self.refperson:
            color = self._config.get("geography.color1")
            self.message_layer.add_message(
                _("Reference : %(name)s ( %(birth)s - %(death)s )")
                % {
                    "name": _nd.display(self.refperson),
                    "birth": self.birth(self.refperson),
                    "death": self.death(self.refperson),
                }
            )
            if indiv1:
                self.message_layer.add_message(
                    _("The other : %(name)s ( %(birth)s - %(death)s )")
                    % {
                        "name": _nd.display(indiv1),
                        "birth": self.birth(indiv1),
                        "death": self.death(indiv1),
                    }
                )
            else:
                self.message_layer.add_message(_("The other person is unknown"))
            self._createmap(self.refperson, color, self.place_list_ref, True)
            if self.refperson_bookmark is None:
                self.refperson_bookmark = self.refperson.get_handle()
                self.add_bookmark_from_popup(None, self.refperson_bookmark)
        else:
            self.message_layer.add_message(_("You must choose one reference person."))
            self.message_layer.add_message(
                _(
                    "Go to the person view and select "
                    "the people you want to compare. "
                    "Return to this view and use the"
                    " history."
                )
            )
        self.possible_meeting(self.place_list_ref, self.place_list_active)
        self.uistate.modify_statusbar(self.dbstate)

    def birth(self, person):
        """
        return "" or the birth date of the person
        """
        birth = get_birth_or_fallback(self.dbstate.db, person)
        if birth and birth.get_type() != EventType.BIRTH:
            sdate = get_date(birth)
            if sdate:
                bdate = "<i>%s</i>" % escape(sdate)
            else:
                bdate = ""
        elif birth:
            bdate = escape(get_date(birth))
        else:
            bdate = ""
        return bdate

    def death(self, person):
        """
        return "" or the death date of the person
        """
        death = get_death_or_fallback(self.dbstate.db, person)
        if death and death.get_type() != EventType.DEATH:
            sdate = get_date(death)
            if sdate:
                ddate = "<i>%s</i>" % escape(sdate)
            else:
                ddate = ""
        elif death:
            ddate = escape(get_date(death))
        else:
            ddate = ""
        return ddate

    def define_actions(self):
        """
        Define action for the reference person button.
        """
        GeoGraphyView.define_actions(self)
        self._add_action("RefPerson", self.select_person)

    def select_person(self, *obj):
        """
        Open a selection box to choose the ref person.
        """
        self.track = []
        self.skip_list = []
        self.refperson = None
        self.refperson_bookmark = None
        selectperson = SelectorFactory("Person")
        sel = selectperson(
            self.dbstate,
            self.uistate,
            self.track,
            _("Select the person which will be our reference."),
            skip=self.skip_list,
        )
        self.refperson = sel.run()
        self.goto_handle(None)

    def build_tree(self):
        """
        This is called by the parent class when the view becomes visible. Since
        all handling of visibility is now in rebuild_trees, see that for more
        information.
        """
        if self.osm is None:
            return
        self.lifeway_layer.clear_ways()
        if not self.dbstate.is_open():
            return
        active = self.get_active()
        if active:
            person = self.dbstate.db.get_person_from_handle(active)
            if person is None:
                self.goto_handle(None)
            else:
                self.goto_handle(handle=person)
        else:
            self.goto_handle(None)

    def draw(self, menu, marks, color, reference):
        """
        Create all moves for the people's event.
        """
        points = []
        mark = None
        for mark in marks:
            startlat = float(mark[3])
            startlon = float(mark[4])
            not_stored = True
            for idx in range(0, len(points)):
                if points[idx][0] == startlat and points[idx][1] == startlon:
                    not_stored = False
            if not_stored:
                points.append((startlat, startlon))
        self.lifeway_layer.add_way(points, color)
        if reference:
            self.lifeway_layer.add_way_ref(
                points,
                "orange",
                float(self._config.get("geography.maximum_meeting_zone")) / 10,
            )
        return False

    def possible_meeting(self, place_list_ref, place_list_active):
        """
        Try to see if two persons can be to the same place during their life.
        If yes, show a marker with the dates foe each person.
        """
        radius = float(self._config.get("geography.maximum_meeting_zone") / 10.0)
        for ref in place_list_ref:
            for act in place_list_active:
                if (
                    hypot(float(act[3]) - float(ref[3]), float(act[4]) - float(ref[4]))
                    <= radius
                ):
                    # we are in the meeting zone
                    self.add_marker(None, None, act[3], act[4], act[7], True, 1)
                    self.all_place_list.append(act)
                    self.add_marker(None, None, ref[3], ref[4], ref[7], True, 1)
                    self.all_place_list.append(ref)

    def _createmap(self, person, color, place_list, reference):
        """
        Create all markers for each people's event in the database which has
        a lat/lon.
        """
        dbstate = self.dbstate
        self.cal = config.get("preferences.calendar-format-report")
        self.place_list = place_list
        self.place_without_coordinates = []
        self.minlat = self.maxlat = self.minlon = self.maxlon = 0.0
        self.minyear = 9999
        self.maxyear = 0
        latitude = ""
        longitude = ""
        if person is not None:
            # For each event, if we have a place, set a marker.
            for event_ref in person.get_event_ref_list():
                if not event_ref:
                    continue
                event = dbstate.db.get_event_from_handle(event_ref.ref)
                role = event_ref.get_role()
                try:
                    date = event.get_date_object().to_calendar(self.cal)
                except Exception:
                    continue
                eyear = (
                    str("%04d" % date.get_year())
                    + str("%02d" % date.get_month())
                    + str("%02d" % date.get_day())
                )
                place_handle = event.get_place_handle()
                if place_handle:
                    place = dbstate.db.get_place_from_handle(place_handle)
                    if place:
                        longitude = place.get_longitude()
                        latitude = place.get_latitude()
                        latitude, longitude = conv_lat_lon(latitude, longitude, "D.D8")
                        descr = _pd.display(dbstate.db, place)
                        evt = EventType(event.get_type())
                        descr1 = _("%(eventtype)s : %(name)s") % {
                            "eventtype": evt,
                            "name": _nd.display(person),
                        }
                        # place.get_longitude and place.get_latitude return
                        # one string. We have coordinates when the two values
                        # contains non null string.
                        if longitude and latitude:
                            self._append_to_places_list(
                                descr,
                                evt,
                                _nd.display(person),
                                latitude,
                                longitude,
                                descr1,
                                eyear,
                                event.get_type(),
                                person.gramps_id,
                                place.gramps_id,
                                event.gramps_id,
                                role,
                            )
                        else:
                            self._append_to_places_without_coord(place.gramps_id, descr)
            family_list = person.get_family_handle_list()
            descr1 = " - "
            for family_hdl in family_list:
                family = self.dbstate.db.get_family_from_handle(family_hdl)
                if family is not None:
                    fhandle = family_list[0]  # first is primary
                    fam = dbstate.db.get_family_from_handle(fhandle)
                    handle = fam.get_father_handle()
                    descr1 = " - "
                    if handle:
                        father = dbstate.db.get_person_from_handle(handle)
                        if father:
                            descr1 = "%s - " % _nd.display(father)
                    handle = fam.get_mother_handle()
                    if handle:
                        mother = dbstate.db.get_person_from_handle(handle)
                        if mother:
                            descr1 = "%s%s" % (descr1, _nd.display(mother))
                    for event_ref in family.get_event_ref_list():
                        if event_ref:
                            event = dbstate.db.get_event_from_handle(event_ref.ref)
                            role = event_ref.get_role()
                            if event.get_place_handle():
                                place_handle = event.get_place_handle()
                                if place_handle:
                                    place = dbstate.db.get_place_from_handle(
                                        place_handle
                                    )
                                    if place:
                                        longitude = place.get_longitude()
                                        latitude = place.get_latitude()
                                        latitude, longitude = conv_lat_lon(
                                            latitude, longitude, "D.D8"
                                        )
                                        descr = _pd.display(dbstate.db, place)
                                        evt = EventType(event.get_type())
                                        eyear = (
                                            str(
                                                "%04d"
                                                % event.get_date_object()
                                                .to_calendar(self.cal)
                                                .get_year()
                                            )
                                            + str(
                                                "%02d"
                                                % event.get_date_object()
                                                .to_calendar(self.cal)
                                                .get_month()
                                            )
                                            + str(
                                                "%02d"
                                                % event.get_date_object()
                                                .to_calendar(self.cal)
                                                .get_day()
                                            )
                                        )
                                        if longitude and latitude:
                                            self._append_to_places_list(
                                                descr,
                                                evt,
                                                _nd.display(person),
                                                latitude,
                                                longitude,
                                                descr1,
                                                eyear,
                                                event.get_type(),
                                                person.gramps_id,
                                                place.gramps_id,
                                                event.gramps_id,
                                                role,
                                            )
                                        else:
                                            self._append_to_places_without_coord(
                                                place.gramps_id, descr
                                            )

            sort1 = sorted(self.place_list, key=operator.itemgetter(6))
            self.draw(None, sort1, color, reference)
            # merge with the last results
            merge_list = []
            for the_list in self.sort, sort1:
                merge_list += the_list
            self.sort = sorted(merge_list, key=operator.itemgetter(6))

    def bubble_message(self, event, lat, lon, marks):
        """
        Create the menu for the selected marker
        """
        self.newmenu = Gtk.Menu()
        menu = self.newmenu
        events = []
        message = ""
        oldplace = ""
        prevmark = None
        for mark in marks:
            for plce in self.all_place_list:
                if plce[3] == mark[3] and plce[4] == mark[4]:
                    if plce[10] in events:
                        continue
                    else:
                        events.append(plce[10])

                    if plce[0] != oldplace:
                        message = "%s :" % plce[0]
                        self.add_place_bubble_message(
                            event, lat, lon, marks, menu, message, plce
                        )
                        oldplace = plce[0]
                        message = ""
                    evt = self.dbstate.db.get_event_from_gramps_id(plce[10])
                    # format the date as described in preferences.
                    date = displayer.display(evt.get_date_object())
                    if date == "":
                        date = _("Unknown")
                    if plce[11] == EventRoleType.PRIMARY:
                        message = "(%s) %s : %s" % (date, plce[2], plce[1])
                    elif plce[11] == EventRoleType.FAMILY:
                        (father_name, mother_name) = self._get_father_and_mother_name(
                            evt
                        )
                        message = "(%s) %s : %s - %s" % (
                            date,
                            plce[7],
                            father_name,
                            mother_name,
                        )
                    else:
                        descr = evt.get_description()
                        if descr == "":
                            descr = _("No description")
                        message = "(%s) %s => %s" % (date, plce[11], descr)
                    prevmark = plce
                    self.add_item = Gtk.MenuItem(label=message)
                    add_item = self.add_item
                    add_item.show()
                    menu.append(add_item)
                    self.itemoption = Gtk.Menu()
                    itemoption = self.itemoption
                    itemoption.show()
                    add_item.set_submenu(itemoption)
                    modify = Gtk.MenuItem(label=_("Edit Event"))
                    modify.show()
                    modify.connect(
                        "activate", self.edit_event, event, lat, lon, prevmark
                    )
                    itemoption.append(modify)
                    center = Gtk.MenuItem(label=_("Center on this place"))
                    center.show()
                    center.connect(
                        "activate", self.center_here, event, lat, lon, prevmark
                    )
                    itemoption.append(center)
                    menu.show()
                    menu.popup_at_pointer(event)
        return 0

    def add_specific_menu(self, menu, event, lat, lon):
        """
        Add specific entry to the navigation menu.
        """
        menu.append(Gtk.SeparatorMenuItem())
        add_item = Gtk.MenuItem(label=_("Choose and bookmark the new reference person"))
        add_item.connect("activate", self.select_person)
        menu.append(add_item)
        return

    def get_default_gramplets(self):
        """
        Define the default gramplets for the sidebar and bottombar.
        """
        return (("Person Filter",), ())

    def specific_options(self, configdialog):
        """
        Add specific entry to the preference menu.
        Must be done in the associated view.
        """
        grid = Gtk.Grid()
        grid.set_border_width(12)
        grid.set_column_spacing(6)
        grid.set_row_spacing(6)
        configdialog.add_text(
            grid,
            _(
                "The meeting zone probability radius.\n"
                "The colored zone is approximative.\n"
                "The meeting zone is only shown for the reference person.\n"
                "The value 9 means about 42 miles or 67 kms.\n"
                "The value 1 means about 4.6 miles or 7.5 kms.\n"
                "The value is in tenth of degree."
            ),
            1,
            line_wrap=False,
        )
        self.config_meeting_slider = configdialog.add_slider(
            grid, "", 2, "geography.maximum_meeting_zone", (1, 9)
        )
        return _("The selection parameters"), grid

    def config_connect(self):
        """
        used to monitor changes in the ini file
        """
        self._config.connect(
            "geography.maximum_meeting_zone", self.cb_update_meeting_radius
        )

    def cb_update_meeting_radius(self, client, cnxn_id, entry, data):
        """
        Called when the radius change
        """
        self.goto_handle(handle=None)



# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2007-2009  Douglas S. Blank <doug.blank@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
AgeOnDateGramplet computes the age for everyone thought to be alive
on a particular date.
"""

# ------------------------------------------------------------------------
#
# Gramps modules
#
# ------------------------------------------------------------------------
from gramps.gen.plug import Gramplet
from gramps.gen.datehandler import parser
from gramps.gui.plug.quick import run_quick_report_by_name
from gramps.gen.const import COLON, GRAMPS_LOCALE as glocale

_ = glocale.translation.sgettext


# ------------------------------------------------------------------------
#
# AgeOnDateGramplet class
#
# ------------------------------------------------------------------------
class AgeOnDateGramplet(Gramplet):
    """
    Gramplet that computes ages on a particular date for everyone
    thought to be alive.
    """

    def init(self):
        """
        Constructs the GUI, consisting of a message, an entry, and
        a Run button.
        """
        from gi.repository import Gtk

        # GUI setup:
        self.set_tooltip(_("Enter a date, click Run"))
        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        hbox = Gtk.Box()
        # label, entry
        description = Gtk.TextView()
        description.set_wrap_mode(Gtk.WrapMode.WORD)
        description.set_editable(False)
        buffer = description.get_buffer()
        buffer.set_text(
            _(
                "Enter a valid date (like YYYY-MM-DD) in the"
                " entry text box and click Run. This will compute"
                " the ages for everyone in your Family Tree on"
                " that date. You can then sort by the age column,"
                " and double-click the row to view or edit."
            )
        )
        label = Gtk.Label()
        label.set_text(_("Date") + COLON)
        self.entry = Gtk.Entry()
        button = Gtk.Button(label=_("Run"))
        button.connect("clicked", self.run)
        ##self.filter =
        vbox.pack_start(hbox, False, True, 0)
        vbox.pack_start(button, False, True, 0)
        hbox.pack_start(label, False, True, 0)
        hbox.pack_start(self.entry, True, True, 0)
        vbox.pack_start(description, True, True, 0)
        self.gui.get_container_widget().remove(self.gui.textview)
        self.gui.get_container_widget().add(vbox)
        vbox.show_all()

    def run(self, obj):
        """
        Method that is run when you click the Run button.
        The date is retrieved from the entry box, parsed as a date,
        and then handed to the quick report.
        """
        text = self.entry.get_text()
        date = parser.parse(text)
        run_quick_report_by_name(self.gui.dbstate, self.gui.uistate, "ageondate", date)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2010  Benny Malengier
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# Standard Python modules
#
# -------------------------------------------------------------------------
from ...const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from . import MatchesFilterBase


# -------------------------------------------------------------------------
#
# Typing modules
#
# -------------------------------------------------------------------------
from ...lib.eventbase import EventBase
from ...db import Database


# -------------------------------------------------------------------------
#
# MatchesEventFilter
#
# -------------------------------------------------------------------------
class MatchesEventFilterBase(MatchesFilterBase):
    """
    Rule that checks against another filter.

    This is a base rule for subclassing by specific objects.
    Subclasses need to define the namespace class attribute.

    """

    labels = ["Event filter name:"]
    name = "Objects with events matching the <event filter>"
    description = "Matches objects who have events that match a certain" " event filter"
    category = _("General filters")

    # we want to have this filter show event filters
    namespace = "Event"

    def prepare(self, db: Database, user):
        MatchesFilterBase.prepare(self, db, user)
        self.MEF_filt = self.find_filter()

    def apply_to_one(self, db: Database, object: EventBase) -> bool:
        if self.MEF_filt is None:
            return False

        eventlist = [x.ref for x in object.event_ref_list]
        for eventhandle in eventlist:
            # check if event in event filter
            event = db.get_event_from_handle(eventhandle)
            if self.MEF_filt.apply_to_one(db, event):
                return True
        return False



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2002-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# Standard Python modules
#
# -------------------------------------------------------------------------
from ....const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from .._matchesfilterbase import MatchesFilterBase


# -------------------------------------------------------------------------
#
# Typing modules
#
# -------------------------------------------------------------------------
from ....lib import Event
from ....db import Database


# -------------------------------------------------------------------------
#
# MatchesFilter
#
# -------------------------------------------------------------------------
class MatchesPlaceFilter(MatchesFilterBase):
    """
    Rule that checks against another filter.

    This is a base rule for subclassing by specific objects.
    Subclasses need to define the namespace class attribute.
    """

    labels = [_("Place filter name:")]
    name = _("Events of places matching the <place filter>")
    description = _(
        "Matches events that occurred at places that match the "
        "specified place filter name"
    )
    category = _("General filters")
    # we want to have this filter show place filters
    namespace = "Place"

    def apply_to_one(self, db: Database, event: Event) -> bool:
        filt = self.find_filter()
        if filt:
            if event and filt.apply_to_one(db, event):
                return True
        return False



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2007  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

import unittest
import logging

from .. import RotateHandler


class RotateHandlerTest(unittest.TestCase):
    """Test the RotateHandler."""

    def test_buffer_recall(self):
        """Test that simple recall of messages works."""

        rh = RotateHandler(10)
        l = logging.getLogger("RotateHandlerTest")
        l.setLevel(logging.DEBUG)
        l.propagate = False

        l.addHandler(rh)

        log_message = "Debug message"
        l.info(log_message)
        self.assertEqual(
            len(rh.get_buffer()),
            1,
            "Message buffer wrong size, should be '1' is '%d'" % (len(rh.get_buffer())),
        )
        self.assertEqual(
            rh.get_buffer()[0].getMessage(),
            log_message,
            "Message buffer content is wrong, should be '%s' is '%s'"
            % (log_message, rh.get_buffer()[0].getMessage()),
        )

        l.removeHandler(rh)

    def test_buffer_rotation(self):
        """Test that buffer correctly rolls over when capacity is reached."""

        rh = RotateHandler(10)
        l = logging.getLogger("RotateHandlerTest")
        l.setLevel(logging.DEBUG)
        l.propagate = False

        l.addHandler(rh)

        log_messages = 20 * [None]
        for i in range(0, 20):
            log_messages[i] = "Message %d" % (i)

        [l.info(log_messages[i]) for i in range(0, 10)]

        self.assertEqual(
            len(rh.get_buffer()),
            10,
            "Message buffer wrong size, should be '10' is '%d'"
            % (len(rh.get_buffer())),
        )

        buffer = rh.get_buffer()

        for i in range(0, 10):
            self.assertEqual(
                buffer[i].getMessage(),
                log_messages[i],
                "Message buffer content is wrong, should be '%s' is '%s'. i = '%d'"
                % (log_messages[i], buffer[i].getMessage(), i),
            )

        l.info(log_messages[10])

        buffer = rh.get_buffer()

        for i in range(0, 10):
            self.assertEqual(
                buffer[i].getMessage(),
                log_messages[i + 1],
                "Message buffer content is wrong, should be '%s' is '%s'. i = '%d'"
                % (log_messages[i + 1], buffer[i].getMessage(), i),
            )

        [l.info(log_messages[i]) for i in range(11, 20)]

        buffer = rh.get_buffer()
        for i in range(0, 10):
            self.assertEqual(
                buffer[i].getMessage(),
                log_messages[i + 10],
                "Message buffer content is wrong, should be '%s' is '%s'. i = '%d'"
                % (log_messages[i + 10], buffer[i].getMessage(), i),
            )

        l.removeHandler(rh)


if __name__ == "__main__":
    unittest.main()



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2007  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# GNOME modules
#
# -------------------------------------------------------------------------
from gi.repository import Gtk

# -------------------------------------------------------------------------
#
# gramps modules
#
# -------------------------------------------------------------------------
from gramps.gen.const import URL_MANUAL_PAGE
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.sgettext
from ._errorreportassistant import ErrorReportAssistant
from ..display import display_help
from ..utils import get_display_size
from ..managedwindow import ManagedWindow

# -------------------------------------------------------------------------
#
# Constants
#
# -------------------------------------------------------------------------
WIKI_HELP_PAGE = "%s_-_Error_and_Warning_Reference" % URL_MANUAL_PAGE
WIKI_HELP_SEC = _("Error_Report", "manual")


class ErrorView(ManagedWindow):
    """
    A Dialog for displaying errors.
    """

    def __init__(self, error_detail, rotate_handler):
        """
        Initialize the handler with the buffer size.
        """
        ManagedWindow.__init__(self, None, [], self.__class__, modal=True)
        # the self.top.run() below makes Gtk make it modal, so any change to
        # the previous line's "modal" would require that line to be changed

        self._error_detail = error_detail
        self._rotate_handler = rotate_handler
        self.p_width = None  # may or may not be changed in draw_window
        self.p_height = None

        self.draw_window()
        self.set_window(self.top, None, None)
        if self.parent_window is not None:
            self.setup_configs(
                "interface.errorview",
                600,
                250,
                # these two may be None or may be real
                p_width=self.p_width,
                p_height=self.p_height,
            )
        if self.p_width is None and self.p_height is None:
            self.show()  # ManagedWindow
        else:
            self.top.show_all()
        self.run()

    def run(self):
        response = Gtk.ResponseType.HELP
        while response == Gtk.ResponseType.HELP:
            # the self.top.run() makes Gtk make it modal, so any change to that
            # line would require the ManagedWindow.__init__ to be changed also
            response = self.top.run()
            if self.parent_window is not None:
                self._save_position(save_config=False)  # the next line saves it
                self._save_size()
            if response == Gtk.ResponseType.HELP:
                self.help_clicked()
            elif response == Gtk.ResponseType.YES:
                ErrorReportAssistant(
                    error_detail=self._error_detail,
                    rotate_handler=self._rotate_handler,
                    ownthread=True,
                    parent=self.top,
                )
                self.top.destroy()
            elif response in (Gtk.ResponseType.CANCEL, Gtk.ResponseType.DELETE_EVENT):
                self.top.destroy()

    def close(self, *obj):
        pass  # let "run" handle it (not ManagedWindow)

    def help_clicked(self):
        """Display the relevant portion of Gramps manual"""

        display_help(WIKI_HELP_PAGE, WIKI_HELP_SEC)

    def draw_window(self):
        title = "%s - Gramps" % _("Error Report")
        self.top = Gtk.Dialog(title=title)
        # look over the top level windows, it seems the oldest come first, so
        # the most recent still visible window appears to be a good choice for
        # a transient parent
        for win in self.top.list_toplevels():
            if win == self.top:  # not interested if this is us...
                continue
            if win.is_toplevel() and win.is_visible():
                self.parent_window = win  # for ManagedWindow
        if self.parent_window is None:  # but it is on some screen
            self.p_width, self.p_height = get_display_size(self.top)
            self.top.set_position(Gtk.WindowPosition.CENTER)
            self.top.set_urgency_hint(True)
            self.top.set_keep_above(True)
            self.top.set_default_size(600, -1)

        vbox = self.top.get_content_area()
        vbox.set_spacing(5)
        self.top.set_border_width(12)
        hbox = Gtk.Box()
        hbox.set_spacing(12)
        image = Gtk.Image()
        image.set_from_icon_name("dialog-error", Gtk.IconSize.DIALOG)
        label = Gtk.Label(
            label='<span size="larger" weight="bold">%s</span>'
            % _("Gramps has experienced an unexpected error")
        )
        label.set_use_markup(True)

        hbox.pack_start(image, False, True, 0)
        hbox.add(label)

        vbox.pack_start(hbox, False, False, 5)

        instructions_label = Gtk.Label(
            label=_(
                "Your data will be safe but it would be advisable to restart Gramps immediately. "
                "If you would like to report the problem to the Gramps team "
                "please click Report and the Error Reporting Wizard will help you "
                "to make a bug report."
            )
        )
        instructions_label.set_line_wrap(True)
        instructions_label.set_use_markup(True)

        vbox.pack_start(instructions_label, False, False, 5)

        tb_frame = Gtk.Frame(label=_("Error Detail"))
        tb_frame.set_border_width(6)
        tb_label = Gtk.TextView()
        tb_label.get_buffer().set_text(self._error_detail.get_formatted_log())
        tb_label.set_border_width(6)
        tb_label.set_editable(False)
        tb_label.set_vexpand(True)

        scroll = Gtk.ScrolledWindow()
        scroll.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
        scroll.set_size_request(-1, 60)

        tb_frame.add(scroll)
        scroll.add(tb_label)

        tb_expander = Gtk.Expander(
            label='<span weight="bold">%s</span>' % _("Error Detail")
        )
        tb_expander.set_use_markup(True)
        tb_expander.add(tb_frame)

        vbox.pack_start(tb_expander, True, True, 5)

        self.top.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
        self.top.add_button(_("Report"), Gtk.ResponseType.YES)
        self.top.add_button(_("_Help"), Gtk.ResponseType.HELP)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
# Copyright (C) 2010       Michiel D. Nauta
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
MediaBase class for Gramps.
"""

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from .const import DIFFERENT, EQUAL, IDENTICAL
from .mediaref import MediaRef


# -------------------------------------------------------------------------
#
# MediaBase
#
# -------------------------------------------------------------------------
class MediaBase:
    """
    Base class for storing media references.
    """

    def __init__(self, source=None):
        """
        Create a new MediaBase, copying from source if not None.

        :param source: Object used to initialize the new object
        :type source: MediaBase
        """
        self.media_list = list(map(MediaRef, source.media_list)) if source else []

    def serialize(self):
        """
        Convert the object to a serialized tuple of data.
        """
        return [mref.serialize() for mref in self.media_list]

    def unserialize(self, data):
        """
        Convert a serialized tuple of data to an object.
        """
        self.media_list = [MediaRef().unserialize(item) for item in data]
        return self

    def add_media_reference(self, media_ref):
        """
        Add a :class:`~.mediaref.MediaRef` instance to the object's media list.

        :param media_ref: :class:`~.mediaref.MediaRef` instance to be added to
                          the object's media list.
        :type media_ref: :class:`~.mediaref.MediaRef`
        """
        self.media_list.append(media_ref)

    def get_media_list(self):
        """
        Return the list of :class:`~.mediaref.MediaRef` instances associated
        with the object.

        :returns: list of :class:`~.mediaref.MediaRef` instances associated
                  with the object
        :rtype: list
        """
        return self.media_list

    def set_media_list(self, media_ref_list):
        """
        Set the list of :class:`~.mediaref.MediaRef` instances associated with
        the object. It replaces the previous list.

        :param media_ref_list: list of :class:`~.mediaref.MediaRef` instances
                               to be assigned to the object.
        :type media_ref_list: list
        """
        self.media_list = media_ref_list

    def _merge_media_list(self, acquisition):
        """
        Merge the list of media references from acquisition with our own.

        :param acquisition: the media list of this object will be merged with
                            the current media reference list.
        :rtype acquisition: MediaBase
        """
        media_list = self.media_list[:]
        for addendum in acquisition.get_media_list():
            for obj in media_list:
                equi = obj.is_equivalent(addendum)
                if equi == IDENTICAL:
                    break
                if equi == EQUAL:
                    obj.merge(addendum)
                    break
            else:
                self.media_list.append(addendum)

    def has_media_reference(self, obj_handle):
        """
        Return True if the object or any of it's child objects has reference
        to this media object handle.

        :param obj_handle: The media handle to be checked.
        :type obj_handle: str
        :returns: Returns whether the object or any of it's child objects has
                  reference to this media handle.
        :rtype: bool
        """
        return obj_handle in [media_ref.ref for media_ref in self.media_list]

    def remove_media_references(self, obj_handle_list):
        """
        Remove references to all media handles in the list.

        :param obj_handle_list: The list of media handles to be removed.
        :type obj_handle_list: list
        """
        new_media_list = [
            media_ref
            for media_ref in self.media_list
            if media_ref.ref not in obj_handle_list
        ]
        self.media_list = new_media_list

    def replace_media_references(self, old_handle, new_handle):
        """
        Replace all references to old media handle with the new handle and
        merge equivalent entries.

        :param old_handle: The media handle to be replaced.
        :type old_handle: str
        :param new_handle: The media handle to replace the old one with.
        :type new_handle: str
        """
        refs_list = [media_ref.ref for media_ref in self.media_list]
        new_ref = None
        if new_handle in refs_list:
            new_ref = self.media_list[refs_list.index(new_handle)]
        n_replace = refs_list.count(old_handle)
        for _ in range(n_replace):
            idx = refs_list.index(old_handle)
            self.media_list[idx].ref = new_handle
            refs_list[idx] = new_handle
            if new_ref:
                media_ref = self.media_list[idx]
                equi = new_ref.is_equivalent(media_ref)
                if equi != DIFFERENT:
                    if equi == EQUAL:
                        new_ref.merge(media_ref)
                    self.media_list.pop(idx)
                    refs_list.pop(idx)



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2004-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Bulgarian-specific classes for parsing and displaying dates.
"""

# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------
import re

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from ..lib.date import Date
from ._dateparser import DateParser
from ._datedisplay import DateDisplay
from ._datehandler import register_datehandler


# -------------------------------------------------------------------------
#
# Bulgarian parser
#
# -------------------------------------------------------------------------
class DateParserBG(DateParser):
    modifier_to_int = {
        "преди": Date.MOD_BEFORE,
        "пр.": Date.MOD_BEFORE,
        "пр": Date.MOD_BEFORE,
        "след": Date.MOD_AFTER,
        "сл.": Date.MOD_AFTER,
        "сл": Date.MOD_AFTER,
        "ок": Date.MOD_ABOUT,
        "ок.": Date.MOD_ABOUT,
        "около": Date.MOD_ABOUT,
        "примерно": Date.MOD_ABOUT,
        "прим": Date.MOD_ABOUT,
        "прим.": Date.MOD_ABOUT,
        "приблизително": Date.MOD_ABOUT,
        "приб.": Date.MOD_ABOUT,
        "прибл.": Date.MOD_ABOUT,
        "приб": Date.MOD_ABOUT,
        "прибл": Date.MOD_ABOUT,
        "from": Date.MOD_FROM,
        "to": Date.MOD_TO,
    }

    calendar_to_int = {
        "григориански": Date.CAL_GREGORIAN,
        "г": Date.CAL_GREGORIAN,
        "юлиански": Date.CAL_JULIAN,
        "ю": Date.CAL_JULIAN,
        "еврейски": Date.CAL_HEBREW,
        "е": Date.CAL_HEBREW,
        "ислямски": Date.CAL_ISLAMIC,
        "и": Date.CAL_ISLAMIC,
        "френски републикански": Date.CAL_FRENCH,
        "републикански": Date.CAL_FRENCH,
        "фр.реп.": Date.CAL_FRENCH,
        "р": Date.CAL_FRENCH,
        "френски": Date.CAL_FRENCH,
        "фр.": Date.CAL_FRENCH,
        "персийски": Date.CAL_PERSIAN,
        "п": Date.CAL_PERSIAN,
    }

    quality_to_int = {
        "приблизително": Date.QUAL_ESTIMATED,
        "прибл.": Date.QUAL_ESTIMATED,
        "изчислено": Date.QUAL_CALCULATED,
        "изчисл.": Date.QUAL_CALCULATED,
        "изч.": Date.QUAL_CALCULATED,
    }

    hebrew_to_int = {
        "тишрей": 1,
        "мархешван": 2,
        "кислев": 3,
        "тевет": 4,
        "шват": 5,
        "адар": 6,
        "адар бет": 7,
        "нисан": 8,
        "ияр": 9,
        "сиван": 10,
        "тамуз": 11,
        "ав": 12,
        "eлул": 13,
    }

    islamic_to_int = {
        "мухаррам": 1,
        "саффар": 2,
        "рабиу-л-ауал": 3,
        "рабиу-с-сани": 4,
        "джумадал-уля": 5,
        "джумада-с-сания": 6,
        "раджаб": 7,
        "шаабан": 8,
        "рамадан": 9,
        "шауал": 10,
        "зу-л-кида": 11,
        "зул-л-хиджа": 12,
    }

    persian_to_int = {
        "фарвардин": 1,
        "урдбихищ": 2,
        "хурдад": 3,
        "тир": 4,
        "мурдад": 5,
        "шахривар": 6,
        "михр": 7,
        "абан": 8,
        "азар": 9,
        "дай": 10,
        "бахман": 11,
        "исфаидармуз": 12,
    }

    french_to_int = {
        "вандемер": 1,
        "брюмер": 2,
        "фример": 3,
        "нивоз": 4,
        "плювиоз": 5,
        "вантоз": 6,
        "жерминал": 7,
        "флореал": 8,
        "прериал": 9,
        "месидор": 10,
        "термидор": 11,
        "фрюктидор": 12,
        "допълнителен": 13,
    }

    bce = ["преди Христа", "пр. Хр.", "пр.Хр."] + DateParser.bce

    def init_strings(self):
        DateParser.init_strings(self)
        _span_1 = ["от"]
        _span_2 = ["до"]
        _range_1 = ["между"]
        _range_2 = ["и"]
        self._span = re.compile(
            r"(%s)\s+(?P<start>.+)\s+(%s)\s+(?P<stop>.+)"
            % ("|".join(_span_1), "|".join(_span_2)),
            re.IGNORECASE,
        )
        self._range = re.compile(
            r"(%s)\s+(?P<start>.+)\s+(%s)\s+(?P<stop>.+)"
            % ("|".join(_range_1), "|".join(_range_2)),
            re.IGNORECASE,
        )


# -------------------------------------------------------------------------
#
# Bulgarian displayer
#
# -------------------------------------------------------------------------
class DateDisplayBG(DateDisplay):
    """
    Bulgarian language date display class.
    """

    long_months = (
        "",
        "януари",
        "февруари",
        "март",
        "април",
        "май",
        "юни",
        "юли",
        "август",
        "септември",
        "октомври",
        "ноември",
        "декември",
    )

    short_months = (
        "",
        "яну",
        "февр",
        "март",
        "апр",
        "май",
        "юни",
        "юли",
        "авг",
        "септ",
        "окт",
        "ное",
        "дек",
    )

    calendar = (
        "",
        "юлиански",
        "еврейски",
        "републикански",
        "персийски",
        "ислямски",
        "шведски",
    )

    _mod_str = ("", "преди ", "след ", "около ", "", "", "")

    _qual_str = ("", "приблизително ", "изчислено ")

    _bce_str = "%s пр. Хр."

    formats = (
        "ГГГГ-ММ-ДД (ISO)",
        "Числов",
        "Месец Ден, Година",
        "Мес. Ден, Година",
        "Ден Месец Година",
        "Ден Мес. Година",
    )
    # this must agree with DateDisplayEn's "formats" definition
    # (since no locale-specific _display_gregorian exists, here)

    hebrew = (
        "",
        "Тишрей",
        "Мархешван",
        "Кислев",
        "Тевет",
        "Шват",
        "Адар",
        "Адар бет",
        "Нисан",
        "Ияр",
        "Сиван",
        "Тамуз",
        "Ав",
        "Елул",
    )

    islamic = (
        "",
        "Мухаррам",
        "Саффар",
        "Рабиу-л-ауал",
        "Рабиу-с-сани",
        "Джумадал-уля",
        "Джумада-с-сания",
        "Раджаб",
        "Шаабан",
        "Рамадан",
        "Шауал",
        "Зу-л-кида",
        "Зул-л-хиджа",
    )

    persian = (
        "",
        "Фарвардин",
        "Урдбихищ",
        "Хурдад",
        "Тир",
        "Мурдад",
        "Шахривар",
        "Михр",
        "Абан",
        "Азар",
        "Дай",
        "Бахман",
        "Исфаидармуз",
    )

    french = (
        "",
        "Вандемер",
        "Брюмер",
        "Фример",
        "Нивоз",
        "Плювиоз",
        "Вантоз",
        "Жерминал",
        "Флореал",
        "Прериал",
        "Мессидор",
        "Термидор",
        "Фрюктидор",
        "Допълнителен",
    )

    def display(self, date):
        """
        Returns a text string representing the date.
        """
        mod = date.get_modifier()
        cal = date.get_calendar()
        qual = date.get_quality()
        start = date.get_start_date()
        newyear = date.get_new_year()

        qual_str = self._qual_str[qual]

        if mod == Date.MOD_TEXTONLY:
            return date.get_text()
        elif start == Date.EMPTY:
            return ""
        elif mod == Date.MOD_SPAN:
            d1 = self.display_cal[cal](start)
            d2 = self.display_cal[cal](date.get_stop_date())
            scal = self.format_extras(cal, newyear)
            return "%s%s %s %s %s%s" % (qual_str, "от", d1, "до", d2, scal)
        elif mod == Date.MOD_RANGE:
            d1 = self.display_cal[cal](start)
            d2 = self.display_cal[cal](date.get_stop_date())
            scal = self.format_extras(cal, newyear)
            return "%s%s %s %s %s%s" % (qual_str, "между", d1, "и", d2, scal)
        else:
            text = self.display_cal[date.get_calendar()](start)
            scal = self.format_extras(cal, newyear)
            return "%s%s%s%s" % (qual_str, self._mod_str[mod], text, scal)

    def dd_dformat01(self, date_val):
        """
        numerical -- for Bulgarian dates
        """
        if date_val[3]:
            return self.display_iso(date_val)
        else:
            if date_val[0] == date_val[1] == 0:
                return str(date_val[2])
            else:
                value = self.dhformat.replace("%m", str(date_val[1]))
                # some locales have %b for the month, e.g. ar_EG, is_IS, nb_NO
                value = value.replace("%b", str(date_val[1]))
                if date_val[0] == 0:  # ignore the zero day and its delimiter
                    i_day = value.find("%e")  # Bulgarian uses %e and not %d
                    value = value.replace(value[i_day : i_day + 3], "")
                value = value.replace("%e", str(date_val[0]))
                value = value.replace("%Y", str(abs(date_val[2])))
                return value.replace("-", "/")


# -------------------------------------------------------------------------
#
# Register classes
#
# -------------------------------------------------------------------------
register_datehandler(
    ("bg_BG", "bg", "bulgarian", "Bulgarian", ("%e.%m.%Y",)),
    DateParserBG,
    DateDisplayBG,
)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2007  Donald N. Allingham
# Copyright (C) 2010       Michiel D. Nauta
# Copyright (C) 2013       Doug Blank <doug.blank@gmail.com>
# Copyright (C) 2017       Nick Hall
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Repository Reference class for Gramps
"""

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------
from ..const import GRAMPS_LOCALE as glocale
from .const import DIFFERENT, EQUAL, IDENTICAL
from .notebase import NoteBase
from .privacybase import PrivacyBase
from .refbase import RefBase
from .secondaryobj import SecondaryObject
from .srcmediatype import SourceMediaType

_ = glocale.translation.gettext


# -------------------------------------------------------------------------
#
# RepoRef
#
# -------------------------------------------------------------------------
class RepoRef(SecondaryObject, PrivacyBase, NoteBase, RefBase):
    """
    Repository reference class.
    """

    def __init__(self, source=None):
        PrivacyBase.__init__(self, source)
        NoteBase.__init__(self, source)
        RefBase.__init__(self, source)
        if source:
            self.call_number = source.call_number
            self.media_type = SourceMediaType(source.media_type)
        else:
            self.call_number = ""
            self.media_type = SourceMediaType()

    def serialize(self):
        """
        Convert the object to a serialized tuple of data.
        """
        return (
            NoteBase.serialize(self),
            RefBase.serialize(self),
            self.call_number,
            self.media_type.serialize(),
            PrivacyBase.serialize(self),
        )

    def unserialize(self, data):
        """
        Convert a serialized tuple of data to an object.
        """
        (note_list, ref, self.call_number, media_type, privacy) = data
        self.media_type = SourceMediaType()
        self.media_type.unserialize(media_type)
        PrivacyBase.unserialize(self, privacy)
        NoteBase.unserialize(self, note_list)
        RefBase.unserialize(self, ref)
        return self

    @classmethod
    def get_schema(cls):
        """
        Returns the JSON Schema for this class.

        :returns: Returns a dict containing the schema.
        :rtype: dict
        """
        return {
            "type": "object",
            "title": _("Repository ref"),
            "properties": {
                "_class": {"enum": [cls.__name__]},
                "note_list": {
                    "type": "array",
                    "title": _("Notes"),
                    "items": {"type": "string", "maxLength": 50},
                },
                "ref": {
                    "type": "string",
                    "title": _("Handle"),
                    "maxLength": 50,
                },
                "call_number": {"type": "string", "title": _("Call Number")},
                "media_type": SourceMediaType.get_schema(),
                "private": {"type": "boolean", "title": _("Private")},
            },
        }

    def get_text_data_list(self):
        """
        Return the list of all textual attributes of the object.

        :returns: Returns the list of all textual attributes of the object.
        :rtype: list
        """
        return [self.call_number, str(self.media_type)]

    def get_referenced_handles(self):
        """
        Return the list of (classname, handle) tuples for all directly
        referenced primary objects.

        :returns: List of (classname, handle) tuples for referenced objects.
        :rtype: list
        """
        ret = self.get_referenced_note_handles()
        if self.ref:
            ret += [("Repository", self.ref)]
        return ret

    def is_equivalent(self, other):
        """
        Return if this repository reference is equivalent, that is agrees in
        reference, call number and medium, to other.

        :param other: The repository reference to compare this one to.
        :type other: RepoRef
        :returns: Constant indicating degree of equivalence.
        :rtype: int
        """
        if (
            self.ref != other.ref
            or self.get_text_data_list() != other.get_text_data_list()
        ):
            return DIFFERENT
        if self.is_equal(other):
            return IDENTICAL
        return EQUAL

    def merge(self, acquisition):
        """
        Merge the content of acquisition into this repository reference.

        :param acquisition: The repository reference to merge with the present
                            repository reference.
        :type acquisition: RepoRef
        """
        self._merge_privacy(acquisition)
        self._merge_note_list(acquisition)

    def set_call_number(self, number):
        """
        Set the call number.
        """
        self.call_number = number

    def get_call_number(self):
        """
        Get the call number.
        """
        return self.call_number

    def get_media_type(self):
        """
        Get the media type.
        """
        return self.media_type

    def set_media_type(self, media_type):
        """
        Set the media type.
        """
        self.media_type.set(media_type)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2001-2006  Donald N. Allingham
# Copyright (C) 2009 Benny Malengier
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Provides the startcli function, which the main program calls for CLI
execution of Gramps.

Provides also two small base classes: :class:`CLIDbLoader`, :class:`CLIManager`
"""

# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext
import os
import sys

import logging

LOG = logging.getLogger(".grampscli")
# -------------------------------------------------------------------------
#
# Gramps  modules
#
# -------------------------------------------------------------------------
from gramps.gen.display.name import displayer as name_displayer
from gramps.gen.config import config
from gramps.gen.const import PLUGINS_DIR, USER_PLUGINS
from gramps.gen.db.dbconst import DBBACKEND
from gramps.gen.db.utils import make_database
from gramps.gen.errors import DbError
from gramps.gen.dbstate import DbState
from gramps.gen.db.exceptions import (
    DbUpgradeRequiredError,
    DbSupportedError,
    DbVersionError,
    DbPythonError,
    DbConnectionError,
)
from gramps.gen.plug import BasePluginManager
from gramps.gen.utils.config import get_researcher
from gramps.gen.recentfiles import recent_files
from gramps.gen.filters import reload_custom_filters


# -------------------------------------------------------------------------
#
# CLI DbLoader class
#
# -------------------------------------------------------------------------
class CLIDbLoader:
    """
    Base class for Db loading action inside a :class:`.DbState`. Only the
    minimum is present needed for CLI handling
    """

    def __init__(self, dbstate):
        self.dbstate = dbstate

    def _warn(self, title, warnmessage):
        """
        Issue a warning message. Inherit for GUI action
        """
        print(_("WARNING: %s") % warnmessage, file=sys.stderr)

    def _errordialog(self, title, errormessage):
        """
        Show the error. A title for the error and an errormessage
        Inherit for GUI action
        """
        print(_("ERROR: %s") % errormessage, file=sys.stderr)
        sys.exit(1)

    def _dberrordialog(self, msg):
        """
        Show a database error.

        :param msg: an error message
        :type msg : string

        .. note:: Inherit for GUI action
        """
        self._errordialog(
            "",
            _("Low level database corruption detected")
            + "\n"
            + _(
                "Gramps has detected a problem in the underlying "
                "database. This can sometimes be repaired from "
                "the Family Tree Manager. Select the database and "
                "click on the Repair button"
            )
            + "\n\n"
            + str(msg),
        )

    def _begin_progress(self):
        """
        Convenience method to allow to show a progress bar if wanted on load
        actions. Inherit if needed
        """
        pass

    def _pulse_progress(self, value):
        """
        Convenience method to allow to show a progress bar if wanted on load
        actions. Inherit if needed
        """
        pass

    def _end_progress(self):
        """
        Convenience method to allow to hide the progress bar if wanted at
        end of load actions. Inherit if needed
        """
        pass

    def read_file(self, filename, username, password):
        """
        This method takes care of changing database, and loading the data.
        In 3.0 we only allow reading of real databases of filetype
        'x-directory/normal'

        This method should only return on success.
        Returning on failure makes no sense, because we cannot recover,
        since database has already been changed.
        Therefore, any errors should raise exceptions.

        On success, return with the disabled signals. The post-load routine
        should enable signals, as well as finish up with other UI goodies.
        """

        if os.path.exists(filename):
            if not os.access(filename, os.W_OK):
                mode = "r"
                self._warn(
                    _("Read only database"),
                    _("You do not have write access " "to the selected file."),
                )
            else:
                mode = "w"
        else:
            mode = "w"

        dbid_path = os.path.join(filename, DBBACKEND)
        if os.path.isfile(dbid_path):
            with open(dbid_path) as file:
                dbid = file.read().strip()
        else:
            dbid = "bsddb"

        db = make_database(dbid)

        self.dbstate.change_database(db)
        self.dbstate.db.disable_signals()

        self._begin_progress()

        try:
            self.dbstate.db.load(
                filename,
                self._pulse_progress,
                mode,
                username=username,
                password=password,
            )
        except (
            DbConnectionError,
            DbSupportedError,
            DbUpgradeRequiredError,
            DbVersionError,
            DbPythonError,
            DbConnectionError,
        ) as msg:
            self.dbstate.no_database()
            self._errordialog(_("Cannot open database"), str(msg))
        except OSError as msg:
            self.dbstate.no_database()
            self._errordialog(_("Could not open file: %s") % filename, str(msg))
        except DbError as msg:
            self.dbstate.no_database()
            self._dberrordialog(msg)
        except Exception:
            self.dbstate.no_database()
            LOG.error("Failed to open database.", exc_info=True)
        return True


# -------------------------------------------------------------------------
#
# CLIManager class
#
# -------------------------------------------------------------------------


class CLIManager:
    """
    Sessionmanager for Gramps.
    This is in effect a reduced :class:`.ViewManager` instance
    (see gui/viewmanager), suitable for CLI actions.
    Aim is to manage a dbstate on which to work (load, unload), and interact
    with the plugin session
    """

    def __init__(self, dbstate, setloader, user):
        self.dbstate = dbstate
        if setloader:
            self.db_loader = CLIDbLoader(self.dbstate)
        else:
            self.db_loader = None
        self.file_loaded = False
        self._pmgr = BasePluginManager.get_instance()
        self.user = user

    def open_activate(self, path, username=None, password=None):
        """
        Open and make a family tree active
        """
        self._read_recent_file(path, username, password)

    def _errordialog(self, title, errormessage):
        """
        Show the error. A title for the error and an errormessage
        """
        print(_("ERROR: %s") % errormessage, file=sys.stderr)
        sys.exit(1)

    def _read_recent_file(self, filename, username=None, password=None):
        """
        Called when a file needs to be loaded
        """
        # A recent database should already have a directory.
        # If not, do nothing, just return.
        # This can be handled better if family tree delete/rename
        # also updated the recent file menu info in displaystate.py
        if not os.path.isdir(filename):
            self._errordialog(
                _("Could not load a recent Family Tree."),
                _("Family Tree does not exist, as it has been deleted."),
            )
            return

        if os.path.isfile(os.path.join(filename, "lock")):
            self._errordialog(
                _("The database is locked."),
                _(
                    "Use the --force-unlock option if you are sure "
                    "that the database is not in use."
                ),
            )
            return

        if self.db_loader.read_file(filename, username, password):
            # Attempt to figure out the database title
            path = os.path.join(filename, "name.txt")
            try:
                with open(path, encoding="utf8") as ifile:
                    title = ifile.readline().strip()
            except:
                title = filename

            self._post_load_newdb(filename, "x-directory/normal", title)

    def _post_load_newdb(self, filename, filetype, title=None):
        """
        The method called after load of a new database.
        Here only CLI stuff is done, inherit this method to add extra stuff
        """
        self._post_load_newdb_nongui(filename, title)

    def _post_load_newdb_nongui(self, filename, title=None):
        """
        Called after a new database is loaded.
        """
        if not filename:
            return

        if filename[-1] == os.path.sep:
            filename = filename[:-1]
        name = os.path.basename(filename)
        self.dbstate.db.db_name = title
        if title:
            name = title

        # apply preferred researcher if loaded file has none
        res = self.dbstate.db.get_researcher()
        owner = get_researcher()
        # If the DB Owner Info is empty and
        # [default] Researcher is not empty and
        # database is empty, then copy default researcher to DB owner
        if res.is_empty() and not owner.is_empty() and self.dbstate.db.get_total() == 0:
            self.dbstate.db.set_researcher(owner)

        name_displayer.clear_custom_formats()
        name_displayer.set_name_format(self.dbstate.db.name_formats)
        fmt_default = config.get("preferences.name-format")
        name_displayer.set_default_format(fmt_default)

        self.dbstate.db.enable_signals()
        self.dbstate.signal_change()

        config.set("paths.recent-file", filename)

        recent_files(filename, name)
        self.file_loaded = True

    def do_reg_plugins(self, dbstate, uistate, rescan=False):
        """
        Register the plugins at initialization time.
        """
        self._pmgr.reg_plugins(PLUGINS_DIR, dbstate, uistate, rescan=rescan)
        self._pmgr.reg_plugins(USER_PLUGINS, dbstate, uistate, load_on_reg=True)
        if rescan:  # supports updated plugin installs
            self._pmgr.reload_plugins()


def startcli(errors, argparser):
    """
    Starts a cli session of Gramps.

    :param errors: errors already encountered
    :param argparser: :class:`.ArgParser` instance
    """
    if errors:
        # already errors encountered. Show first one on terminal and exit
        errmsg = _("Error encountered: %s") % errors[0][0]
        print(errmsg, file=sys.stderr)
        errmsg = _("  Details: %s") % errors[0][1]
        print(errmsg, file=sys.stderr)
        sys.exit(1)

    if argparser.errors:
        errmsg = _("Error encountered in argument parsing: %s") % argparser.errors[0][0]
        print(errmsg, file=sys.stderr)
        errmsg = _("  Details: %s") % argparser.errors[0][1]
        print(errmsg, file=sys.stderr)
        sys.exit(1)

    # we need to keep track of the db state
    dbstate = DbState()

    # we need a manager for the CLI session
    from .user import User

    user = User(auto_accept=argparser.auto_accept, quiet=argparser.quiet)
    climanager = CLIManager(dbstate, True, user)

    # load the plugins
    climanager.do_reg_plugins(dbstate, uistate=None)
    reload_custom_filters()
    # handle the arguments
    from .arghandler import ArgHandler

    handler = ArgHandler(dbstate, argparser, climanager)
    # create a manager to manage the database

    handler.handle_args_cli()
    if handler.dbstate.is_open():
        handler.dbstate.db.close()
    sys.exit(0)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
Provide the interface to allow a person to add a media object to the database.
"""

# -------------------------------------------------------------------------
#
# Standard python modules
#
# -------------------------------------------------------------------------
import os

# -------------------------------------------------------------------------
#
# internationalization
#
# -------------------------------------------------------------------------

# -------------------------------------------------------------------------
#
# GTK/Gnome modules
#
# -------------------------------------------------------------------------
from gi.repository import GdkPixbuf

# -------------------------------------------------------------------------
#
# gramps modules
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.sgettext
from gramps.gen.const import ICON, THUMBSCALE, USER_HOME
from gramps.gen.config import config
from gramps.gen.utils.file import media_path_full, media_path, relative_path, find_file
from gramps.gen.mime import get_type
from gramps.gen.utils.thumbnails import get_thumbnail_image
from ..display import display_help
from ..managedwindow import ManagedWindow
from ..dialog import ErrorDialog, WarningDialog
from ..glade import Glade
from gramps.gen.const import URL_MANUAL_SECT2

# -------------------------------------------------------------------------
#
# Constants
#
# -------------------------------------------------------------------------
WIKI_HELP_PAGE = URL_MANUAL_SECT2
WIKI_HELP_SEC = _("Select_a_media_selector", "manual")


# -------------------------------------------------------------------------
#
# AddMedia
#
# -------------------------------------------------------------------------
class AddMedia(ManagedWindow):
    """
    Displays the Add Media Dialog window, allowing the user to select
    a file from the file system, while providing a description.
    """

    def __init__(self, dbstate, uistate, track, media, callback=None):
        """
        Create and displays the dialog box

        db - the database in which the new object is to be stored
        The media is updated with the information, and on save, the
        callback function is called
        """
        ManagedWindow.__init__(self, uistate, track, self, modal=True)

        self.dbase = dbstate.db
        self.obj = media
        self.callback = callback

        self.last_directory = config.get("behavior.addmedia-image-dir")
        self.relative_path = config.get("behavior.addmedia-relative-path")

        self.glade = Glade()
        self.set_window(
            self.glade.toplevel,
            self.glade.get_object("title"),
            _("Select a media object"),
        )
        self.setup_configs("interface.addmedia", 700, 500)

        self.description = self.glade.get_object("photoDescription")
        self.image = self.glade.get_object("image")
        self.file_text = self.glade.get_object("fname")
        if not (self.last_directory and os.path.isdir(self.last_directory)):
            self.last_directory = USER_HOME
        # if existing path, use dir of path
        if not self.obj.get_path() == "":
            fullname = media_path_full(self.dbase, self.obj.get_path())
            dir = os.path.dirname(fullname)
            if os.path.isdir(dir):
                self.last_directory = dir
                self.file_text.select_filename(fullname)
            else:
                self.file_text.set_current_folder(self.last_directory)
        else:
            self.file_text.set_current_folder(self.last_directory)
        if not self.obj.get_description() == "":
            self.description.set_text(self.obj.get_description())

        self.relpath = self.glade.get_object("relpath")
        self.relpath.set_active(self.relative_path)
        self.temp_name = ""
        self.object = None

        self.glade.get_object("fname").connect("update_preview", self.on_name_changed)
        self.ok_button = self.glade.get_object("button79")
        self.help_button = self.glade.get_object("button103")
        self.cancel_button = self.glade.get_object("button81")
        self.ok_button.connect("clicked", self.save)
        self.ok_button.set_sensitive(not self.dbase.readonly)
        self.help_button.connect(
            "clicked",
            lambda x: display_help(webpage=WIKI_HELP_PAGE, section=WIKI_HELP_SEC),
        )
        self.cancel_button.connect("clicked", self.close)
        self.show()

    def build_menu_names(self, obj):
        """
        Build the menu name for the window manager.
        """
        return (_("Select media object"), None)

    def save(self, *obj):
        """
        Callback function called when the save button is pressed.
        The media object is updated, and callback called.
        """
        description = str(self.description.get_text())

        if self.file_text.get_filename() is None:
            msgstr = _("Import failed")
            msgstr2 = _("The filename supplied could not be found.")
            ErrorDialog(msgstr, msgstr2, parent=self.window)
            return

        filename = self.file_text.get_filename()
        full_file = filename

        if self.relpath.get_active():
            pname = str(media_path(self.dbase))
            if not os.path.exists(pname):
                msgstr = _("Cannot import %s")
                msgstr2 = _(
                    "Directory specified in preferences: "
                    "Base path for relative media paths: "
                    "%s does not exist. Change preferences "
                    "or do not use relative path when importing"
                )
                ErrorDialog(msgstr % filename, msgstr2 % pname, parent=self.window)
                return
            filename = relative_path(filename, pname)

        mtype = get_type(full_file)
        description = description or os.path.basename(filename)

        self.obj.set_description(description)
        self.obj.set_mime_type(mtype)
        name = filename
        self.obj.set_path(name)

        self.last_directory = os.path.dirname(full_file)
        self.relative_path = self.relpath.get_active()

        self._cleanup_on_exit()
        if self.callback:
            self.callback(self.obj)
        self.close()

    def on_name_changed(self, *obj):
        """
        Called anytime the filename text window changes. Checks to
        see if the file exists. If it does, the image is loaded into
        the preview window.
        """
        fname = self.file_text.get_filename()
        if not fname:
            return
        filename = fname
        basename = os.path.basename(filename)
        (root, ext) = os.path.splitext(basename)
        old_title = self.description.get_text()

        if old_title == "" or old_title == self.temp_name:
            self.description.set_text(root)
        self.temp_name = root

        filename = find_file(filename)
        if filename:
            mtype = get_type(filename)
            image = get_thumbnail_image(filename, mtype)
            self.image.set_from_pixbuf(image)

    def _cleanup_on_exit(self):
        config.set("behavior.addmedia-image-dir", self.last_directory)
        config.set("behavior.addmedia-relative-path", self.relative_path)
        config.save()

    # -------------------------------------------------------------------------
    #
    # scale_image
    #
    # -------------------------------------------------------------------------
    def scale_image(self, path, size):
        """
        Scales the image to the specified size
        """

        title_msg = _("Cannot display %s") % path
        detail_msg = _(
            "Gramps is not able to display the image file. "
            "This may be caused by a corrupt file."
        )

        try:
            image1 = GdkPixbuf.Pixbuf.new_from_file(path)
            width = image1.get_width()
            height = image1.get_height()
            scale = size / float(max(width, height))
            return image1.scale_simple(
                int(scale * width), int(scale * height), GdkPixbuf.InterpType.BILINEAR
            )
        except:
            WarningDialog(title_msg, detail_msg, parent=self.window)
            return GdkPixbuf.Pixbuf.new_from_file(ICON)



# -*- coding: utf-8 -*-
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2003-2007  Donald N. Allingham
# Copyright (C) 2008-2010  Brian G. Matherly
# Copyright (C) 2007-2010  Jerome Rapinat
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# -------------------------------------------------------------------------
"""
French-specific classes for relationships.
"""

# -------------------------------------------------------------------------
#
# Gramps modules
#
# -------------------------------------------------------------------------

from gramps.gen.lib import Person
import gramps.gen.relationship

# -------------------------------------------------------------------------

# level is used for generation level:
# at %th generation

_LEVEL_NAME = [
    "première",
    "deuxième",
    "troisième",
    "quatrième",
    "cinquième",
    "sixième",
    "septième",
    "huitième",
    "neuvième",
    "dixième",
    "onzième",
    "douzième",
    "treizième",
    "quatorzième",
    "quinzième",
    "seizième",
    "dix-septième",
    "dix-huitième",
    "dix-neuvième",
    "vingtième",
    "vingt-et-unième",
    "vingt-deuxième",
    "vingt-troisième",
    "vingt-quatrième",
    "vingt-cinquième",
    "vingt-sixième",
    "vingt-septième",
    "vingt-huitième",
    "vingt-neuvième",
    "trentième",
]

# for degree (canon et civil), limitation 20+20 also used for
# the first [premier] cousin

_REMOVED_LEVEL = [
    "premier",
    "deuxième",
    "troisième",
    "quatrième",
    "cinquième",
    "sixième",
    "septième",
    "huitième",
    "neuvième",
    "dixième",
    "onzième",
    "douzième",
    "treizième",
    "quatorzième",
    "quinzième",
    "seizième",
    "dix-septième",
    "dix-huitième",
    "dix-neuvième",
    "vingtième",
    "vingt-et-unième",
    "vingt-deuxième",
    "vingt-troisième",
    "vingt-quatrième",
    "vingt-cinquième",
    "vingt-sixième",
    "vingt-septième",
    "vingt-huitième",
    "vingt-neuvième",
    "trentième",
    "trente-et-unième",
    "trente-deuxième",
    "trente-troisième",
    "trente-quatrième",
    "trente-cinquième",
    "trente-sixième",
    "trente-septième",
    "trente-huitième",
    "trente-neuvième",
    "quarantième",
    "quanrante-et-unième",
]

# small lists, use generation level if > [5]

_FATHER_LEVEL = [
    "",
    "le père%s",
    "le grand-père%s",
    "l'arrière-grand-père%s",
    "le trisaïeul%s",
]

_MOTHER_LEVEL = [
    "",
    "la mère%s",
    "la grand-mère%s",
    "l'arrière-grand-mère%s",
    "la trisaïeule%s",
]

_SON_LEVEL = ["", "le fils%s", "le petit-fils%s", "l'arrière-petit-fils%s"]

_DAUGHTER_LEVEL = ["", "la fille%s", "la petite-fille%s", "l'arrière-petite-fille%s"]

_SISTER_LEVEL = [
    "",
    "la sœur%s",
    "la tante%s",
    "la grand-tante%s",
    "l'arrière-grand-tante%s",
]

_BROTHER_LEVEL = [
    "",
    "le frère%s",
    "l'oncle%s",
    "le grand-oncle%s",
    "l'arrière-grand-oncle%s",
]

_NEPHEW_LEVEL = ["", "le neveu%s", "le petit-neveu%s", "l'arrière-petit-neveu%s"]

_NIECE_LEVEL = ["", "la nièce%s", "la petite-nièce%s", "l'arrière-petite-nièce%s"]

# kinship report

_PARENTS_LEVEL = [
    "",
    "les parents",
    "les grands-parents",
    "les arrières-grands-parents",
    "les trisaïeux",
]

_CHILDREN_LEVEL = [
    "",
    "les enfants",
    "les petits-enfants",
    "les arrières-petits-enfants",
    "les arrières-arrières-petits-enfants",
]

_SIBLINGS_LEVEL = [
    "",
    "les frères et les sœurs",
    "les oncles et les tantes",
    "les grands-oncles et les grands-tantes",
    "les arrières-grands-oncles et les arrières-grands-tantes",
]

_NEPHEWS_NIECES_LEVEL = [
    "",
    "les neveux et les nièces",
    "les petits-neveux et les petites-nièces",
    "les arrière-petits-neveux et les arrières-petites-nièces",
]

# -------------------------------------------------------------------------
#
#
#
# -------------------------------------------------------------------------

# from active person to common ancestor Ga=[level]


def get_cousin(level, removed, inlaw=""):
    """
    cousins = same level, gender = male
    """
    if removed == 0 and level < len(_LEVEL_NAME):
        return "le %s cousin%s" % (_REMOVED_LEVEL[level - 1], inlaw)
    elif level < removed:
        get_uncle(level - 1, inlaw)
    elif level < 30:
        # limitation gen = 30

        return "le cousin lointain, relié à la %s génération" % _LEVEL_NAME[removed]
    else:
        # use numerical generation

        return "le cousin lointain, relié à la %dème génération" % (level + 1)


def get_cousine(level, removed, inlaw=""):
    """
    cousines = same level, gender = female
    """
    if removed == 0 and level < len(_LEVEL_NAME):
        return "la %s cousine%s" % (_LEVEL_NAME[level - 1], inlaw)
    elif level < removed:
        get_aunt(level - 1, inlaw)
    elif level < 30:
        # limitation gen = 30

        return "la cousine lointaine, reliée à la %s génération" % _LEVEL_NAME[removed]
    else:
        # use numerical generation

        return "la cousine lointaine, reliée à la %dème génération" % (level + 1)


def get_parents(level):
    """
    ancestors
    """
    if level > len(_PARENTS_LEVEL) - 1 and level < 30:
        # limitation gen = 30

        return "les ascendants lointains, à la %s génération" % _LEVEL_NAME[level]
    elif level > len(_PARENTS_LEVEL) - 1:
        # use numerical generation

        return "les ascendants lointains, à la %dème génération" % level
    else:
        return _PARENTS_LEVEL[level]


def get_father(level, inlaw=""):
    """
    ancestor, gender = male
    """
    if level > len(_FATHER_LEVEL) - 1 and level < 30:
        # limitation gen = 30

        return "l'ascendant lointain, à la %s génération" % _LEVEL_NAME[level]
    elif level > len(_FATHER_LEVEL) - 1:
        # use numerical generation

        return "l'ascendant lointain, à la %dème génération" % level
    else:
        return _FATHER_LEVEL[level] % inlaw


def get_mother(level, inlaw=""):
    """
    ancestor, gender = female
    """
    if level > len(_MOTHER_LEVEL) - 1 and level < 30:
        # limitation gen = 30

        return "l'ascendante lointaine, à la %s génération" % _LEVEL_NAME[level]
    elif level > len(_MOTHER_LEVEL) - 1:
        # use numerical generation

        return "l'ascendante lointaine, à la %dème génération" % level
    else:
        return _MOTHER_LEVEL[level] % inlaw


def get_parent_unknown(level, inlaw=""):
    """
    unknown parent
    """
    if level > len(_LEVEL_NAME) - 1 and level < 30:
        # limitation gen = 30

        return "l'ascendant lointain, à la %s génération" % _LEVEL_NAME[level]
    elif level == 1:
        return "un parent%s" % inlaw
    else:
        return "un parent lointain%s" % inlaw


def get_son(level, inlaw=""):
    """
    descendant, gender = male
    """
    if level > len(_SON_LEVEL) - 1 and level < 30:
        # limitation gen = 30

        return "le descendant lointain, à la %s génération" % _LEVEL_NAME[level + 1]
    elif level > len(_SON_LEVEL) - 1:
        # use numerical generation

        return "le descendant lointain, à la %dème génération" % level
    else:
        return _SON_LEVEL[level] % inlaw


def get_daughter(level, inlaw=""):
    """
    descendant, gender = female
    """
    if level > len(_DAUGHTER_LEVEL) - 1 and level < 30:
        # limitation gen = 30

        return "la descendante lointaine, à la %s génération" % _LEVEL_NAME[level + 1]
    elif level > len(_DAUGHTER_LEVEL) - 1:
        # use numerical generation

        return "la descendante lointaine, à la %dème génération" % level
    else:
        return _DAUGHTER_LEVEL[level] % inlaw


def get_child_unknown(level, inlaw=""):
    """
    descendant, gender = unknown
    """
    if level > len(_LEVEL_NAME) - 1 and level < 30:
        # limitation gen = 30

        return "le descendant lointain, à la %s génération" % _LEVEL_NAME[level + 1]
    elif level == 1:
        return "un enfant%s" % inlaw
    else:
        return "un descendant lointain%s" % inlaw


def get_sibling_unknown(inlaw=""):
    """
    sibling of an ancestor, gender = unknown
    """
    return "un parent lointain%s" % inlaw


def get_uncle(level, inlaw=""):
    """
    sibling of an ancestor, gender = male
    """
    if level > len(_BROTHER_LEVEL) - 1 and level < 30:
        # limitation gen = 30

        return "l'oncle lointain, relié à la %s génération" % _LEVEL_NAME[level]
    elif level > len(_BROTHER_LEVEL) - 1:
        # use numerical generation

        return "l'oncle lointain, relié à la %dème génération" % (level + 1)
    else:
        return _BROTHER_LEVEL[level] % inlaw


def get_aunt(level, inlaw=""):
    """
    sibling of an ancestor, gender = female
    """
    if level > len(_SISTER_LEVEL) - 1 and level < 30:
        # limitation gen = 30

        return "la tante lointaine, reliée à la %s génération" % _LEVEL_NAME[level]
    elif level > len(_SISTER_LEVEL) - 1:
        # use numerical generation

        return "la tante lointaine, reliée à la %dème génération" % (level + 1)
    else:
        return _SISTER_LEVEL[level] % inlaw


def get_nephew(level, inlaw=""):
    """
    cousin of a descendant, gender = male
    """
    if level > len(_NEPHEW_LEVEL) - 1 and level < 30:
        # limitation gen = 30

        return "le neveu lointain, à la %s génération" % _LEVEL_NAME[level]
    elif level > len(_NEPHEW_LEVEL) - 1:
        # use numerical generation

        return "le neveu lointain, à la %dème génération" % (level + 1)
    else:
        return _NEPHEW_LEVEL[level] % inlaw


def get_niece(level, inlaw=""):
    """
    cousin of a descendant, gender = female
    """
    if level > len(_NIECE_LEVEL) - 1 and level < 30:
        # limitation gen = 30

        return "la nièce lointaine, à la %s génération" % _LEVEL_NAME[level]
    elif level > len(_NIECE_LEVEL) - 1:
        # use numerical generation

        return "la nièce lointaine, à la %dème génération" % (level + 1)
    else:
        return _NIECE_LEVEL[level] % inlaw


class RelationshipCalculator(gramps.gen.relationship.RelationshipCalculator):
    """
    RelationshipCalculator Class
    """

    INLAW = " (par alliance)"

    def __init__(self):
        gramps.gen.relationship.RelationshipCalculator.__init__(self)

    # kinship report

    def get_plural_relationship_string(
        self,
        Ga,
        Gb,
        reltocommon_a="",
        reltocommon_b="",
        only_birth=True,
        in_law_a=False,
        in_law_b=False,
    ):
        """
        voir relationship.py
        """

        rel_str = "des parents lointains"
        atgen = " à la %sème génération"
        bygen = " par la %sème génération"
        cmt = " (frères ou sœurs d'un ascendant" + atgen % Ga + ")"
        if Ga == 0:
            # These are descendants

            if Gb < len(_CHILDREN_LEVEL):
                rel_str = _CHILDREN_LEVEL[Gb]
            else:
                rel_str = "les descendants" + atgen % (Gb + 1)
        elif Gb == 0:
            # These are parents/grand parents

            if Ga < len(_PARENTS_LEVEL):
                rel_str = _PARENTS_LEVEL[Ga]
            else:
                rel_str = "les ascendants" + atgen % (Ga + 1)
        elif Gb == 1:
            # These are siblings/aunts/uncles

            if Ga < len(_SIBLINGS_LEVEL):
                rel_str = _SIBLINGS_LEVEL[Ga]
            else:
                rel_str = "les enfants d'un ascendant" + atgen % (Ga + 1) + cmt
        elif Ga == 1:
            # These are nieces/nephews

            if Gb < len(_NEPHEWS_NIECES_LEVEL):
                rel_str = _NEPHEWS_NIECES_LEVEL[Gb - 1]
            else:
                rel_str = "les neveux et les nièces" + atgen % Gb
        elif Ga > 1 and Ga == Gb:
            # These are cousins in the same generation
            # use custom level for latin words

            if Ga == 2:
                rel_str = "les cousins germains et cousines germaines"
            elif Ga <= len(_LEVEL_NAME):
                # %ss for plural

                rel_str = "les %ss cousins et cousines" % _LEVEL_NAME[Ga - 2]
            else:
                # security

                rel_str = "les cousins et cousines"
        elif Ga > 1 and Ga > Gb:
            # These are cousins in different generations with the second person
            # being in a higher generation from the common ancestor than the
            # first person.
            # use custom level for latin words and specific relation

            if Ga == 3 and Gb == 2:
                desc = " (cousins germains d'un parent)"
                rel_str = "les oncles et tantes à la mode de Bretagne" + desc
            elif (
                Gb <= len(_LEVEL_NAME)
                and Ga - Gb < len(_REMOVED_LEVEL)
                and Ga + Gb + 1 < len(_REMOVED_LEVEL)
            ):
                can = " du %s au %s degré (canon)" % (
                    _REMOVED_LEVEL[Gb],
                    _REMOVED_LEVEL[Ga],
                )
                civ = " et au %s degré (civil)" % _REMOVED_LEVEL[Ga + Gb + 1]
                rel_str = "les oncles et tantes" + can + civ
            elif Ga < len(_LEVEL_NAME):
                rel_str = "les grands-oncles et grands-tantes" + bygen % (Ga + 1)

        elif Gb > 1 and Gb > Ga:
            # These are cousins in different generations with the second person
            # being in a lower generation from the common ancestor than the
            # first person.
            # use custom level for latin words and specific relation

            if Ga == 2 and Gb == 3:
                info = " (cousins issus d'un germain)"
                rel_str = "les neveux et nièces à la mode de Bretagne" + info
            elif (
                Ga <= len(_LEVEL_NAME)
                and Gb - Ga < len(_REMOVED_LEVEL)
                and Ga + Gb + 1 < len(_REMOVED_LEVEL)
            ):
                can = " du %s au %s degré (canon)" % (
                    _REMOVED_LEVEL[Gb],
                    _REMOVED_LEVEL[Ga],
                )
                civ = " et au %s degré (civil)" % _REMOVED_LEVEL[Ga + Gb + 1]
                rel_str = "les neveux et nièces" + can + civ
            elif Ga < len(_LEVEL_NAME):
                rel_str = "les neveux et nièces" + bygen % Gb

        if in_law_b:
            rel_str = "les conjoints pour %s" % rel_str

        return rel_str

    # quick report (missing on RelCalc tool - Status Bar)

    def get_single_relationship_string(
        self,
        Ga,
        Gb,
        gender_a,
        gender_b,
        reltocommon_a,
        reltocommon_b,
        only_birth=True,
        in_law_a=False,
        in_law_b=False,
    ):
        """
        voir relationship.py
        """

        if only_birth:
            step = ""
        else:
            step = self.STEP

        if in_law_a or in_law_b:
            inlaw = self.INLAW
        else:
            inlaw = ""

        rel_str = "un parent lointains%s" % inlaw
        bygen = " par la %sème génération"
        if Ga == 0:
            # b is descendant of a

            if Gb == 0:
                rel_str = "le même individu"
            elif gender_b == Person.MALE and Gb < len(_SON_LEVEL):
                # spouse of daughter

                if inlaw and Gb == 1 and not step:
                    rel_str = "le gendre"
                else:
                    rel_str = get_son(Gb)
            elif gender_b == Person.FEMALE and Gb < len(_DAUGHTER_LEVEL):
                # spouse of son

                if inlaw and Gb == 1 and not step:
                    rel_str = "la bru"
                else:
                    rel_str = get_daughter(Gb)
            elif Gb < len(_LEVEL_NAME) and gender_b == Person.MALE:
                # don't display inlaw

                rel_str = "le descendant lointain (%dème génération)" % (Gb + 1)
            elif Gb < len(_LEVEL_NAME) and gender_b == Person.FEMALE:
                rel_str = "la descendante lointaine (%dème génération)" % (Gb + 1)
            else:
                return get_child_unknown(Gb)
        elif Gb == 0:
            # b is parents/grand parent of a

            if gender_b == Person.MALE and Ga < len(_FATHER_LEVEL):
                # other spouse of father (new parent)

                if Ga == 1 and inlaw and self.STEP_SIB:
                    rel_str = "le beau-père"
                elif Ga == 1 and inlaw:
                    # father of spouse (family of spouse)

                    rel_str = "le père du conjoint"
                else:
                    rel_str = get_father(Ga, inlaw)
            elif gender_b == Person.FEMALE and Ga < len(_MOTHER_LEVEL):
                # other spouse of mother (new parent)

                if Ga == 1 and inlaw and self.STEP_SIB:
                    rel_str = "la belle-mère"
                elif Ga == 1 and inlaw:
                    # mother of spouse (family of spouse)

                    rel_str = "la mère du conjoint"
                else:
                    rel_str = get_mother(Ga, inlaw)
            elif Ga < len(_LEVEL_NAME) and gender_b == Person.MALE:
                rel_str = "l'ascendant lointain%s (%dème génération)" % (inlaw, Ga + 1)
            elif Ga < len(_LEVEL_NAME) and gender_b == Person.FEMALE:
                rel_str = "l'ascendante lointaine%s (%dème génération)" % (
                    inlaw,
                    Ga + 1,
                )
            else:
                return get_parent_unknown(Ga, inlaw)
        elif Gb == 1:
            # b is sibling/aunt/uncle of a

            if gender_b == Person.MALE and Ga < len(_BROTHER_LEVEL):
                rel_str = get_uncle(Ga, inlaw)
            elif gender_b == Person.FEMALE and Ga < len(_SISTER_LEVEL):
                rel_str = get_aunt(Ga, inlaw)
            else:
                # don't display inlaw

                if gender_b == Person.MALE:
                    rel_str = "l'oncle lointain" + bygen % (Ga + 1)
                elif gender_b == Person.FEMALE:
                    rel_str = "la tante lointaine" + bygen % (Ga + 1)
                elif gender_b == Person.UNKNOWN:
                    rel_str = get_sibling_unknown(inlaw)
                else:
                    return rel_str
        elif Ga == 1:
            # b is niece/nephew of a

            if gender_b == Person.MALE and Gb < len(_NEPHEW_LEVEL):
                rel_str = get_nephew(Gb - 1, inlaw)
            elif gender_b == Person.FEMALE and Gb < len(_NIECE_LEVEL):
                rel_str = get_niece(Gb - 1, inlaw)
            else:
                if gender_b == Person.MALE:
                    rel_str = "le neveu lointain%s (%dème génération)" % (inlaw, Gb)
                elif gender_b == Person.FEMALE:
                    rel_str = "la nièce lointaine%s (%dème génération)" % (inlaw, Gb)
                elif gender_b == Person.UNKNOWN:
                    rel_str = get_sibling_unknown(inlaw)
                else:
                    return rel_str
        elif Ga == Gb:
            # a and b cousins in the same generation

            if gender_b == Person.MALE:
                rel_str = get_cousin(Ga - 1, 0, inlaw=inlaw)
            elif gender_b == Person.FEMALE:
                rel_str = get_cousine(Ga - 1, 0, inlaw=inlaw)
            elif gender_b == Person.UNKNOWN:
                rel_str = get_sibling_unknown(inlaw)
            else:
                return rel_str
        elif Ga > 1 and Ga > Gb:
            # These are cousins in different generations with the second person
            # being in a higher generation from the common ancestor than the
            # first person.

            if Ga == 3 and Gb == 2:
                if gender_b == Person.MALE:
                    desc = " (cousin germain d'un parent)"
                    rel_str = "l'oncle à la mode de Bretagne" + desc
                elif gender_b == Person.FEMALE:
                    desc = " (cousine germaine d'un parent)"
                    rel_str = "la tante à la mode de Bretagne" + desc
                elif gender_b == Person.UNKNOWN:
                    return get_sibling_unknown(inlaw)
                else:
                    return rel_str
            elif (
                Gb <= len(_LEVEL_NAME)
                and Ga - Gb < len(_REMOVED_LEVEL)
                and Ga + Gb + 1 < len(_REMOVED_LEVEL)
            ):
                can = " du %s au %s degré (canon)" % (
                    _REMOVED_LEVEL[Gb],
                    _REMOVED_LEVEL[Ga],
                )
                civ = " et au %s degré (civil)" % _REMOVED_LEVEL[Ga + Gb + 1]
                if gender_b == Person.MALE:
                    rel_str = "l'oncle" + can + civ
                elif gender_b == Person.FEMALE:
                    rel_str = "la tante" + can + civ
                elif gender_b == Person.UNKNOWN:
                    rel_str = get_sibling_unknown(inlaw)
                else:
                    return rel_str
            else:
                if gender_b == Person.MALE:
                    rel_str = get_uncle(Ga, inlaw)
                elif gender_b == Person.FEMALE:
                    rel_str = get_aunt(Ga, inlaw)
                elif gender_b == Person.UNKNOWN:
                    rel_str = get_sibling_unknown(inlaw)
                else:
                    return rel_str
        elif Gb > 1 and Gb > Ga:
            # These are cousins in different generations with the second person
            # being in a lower generation from the common ancestor than the
            # first person.

            if Ga == 2 and Gb == 3:
                info = " (cousins issus d'un germain)"
                if gender_b == Person.MALE:
                    rel_str = "le neveu à la mode de Bretagne" + info
                elif gender_b == Person.FEMALE:
                    rel_str = "la nièce à la mode de Bretagne" + info
                elif gender_b == Person.UNKNOWN:
                    rel_str = get_sibling_unknown(inlaw)
                else:
                    return rel_str
            elif (
                Ga <= len(_LEVEL_NAME)
                and Gb - Ga < len(_REMOVED_LEVEL)
                and Ga + Gb + 1 < len(_REMOVED_LEVEL)
            ):
                can = " du %s au %s degré (canon)" % (
                    _REMOVED_LEVEL[Gb],
                    _REMOVED_LEVEL[Ga],
                )
                civ = " et au %s degré (civil)" % _REMOVED_LEVEL[Ga + Gb + 1]
                if gender_b == Person.MALE:
                    rel_str = "le neveu" + can + civ
                if gender_b == Person.FEMALE:
                    rel_str = "la nièce" + can + civ
                elif gender_b == Person.UNKNOWN:
                    rel_str = get_sibling_unknown(inlaw)
                else:
                    return rel_str
            elif Ga > len(_LEVEL_NAME):
                return rel_str
            else:
                if gender_b == Person.MALE:
                    rel_str = get_nephew(Ga, inlaw)
                elif gender_b == Person.FEMALE:
                    rel_str = get_niece(Ga, inlaw)
                elif gender_b == Person.UNKNOWN:
                    rel_str = get_sibling_unknown(inlaw)
                else:
                    return rel_str
        return rel_str

    # RelCalc tool - Status Bar

    def get_sibling_relationship_string(
        self, sib_type, gender_a, gender_b, in_law_a=False, in_law_b=False
    ):
        """
        voir relationship.py
        """

        if in_law_a or in_law_b:
            inlaw = self.INLAW
        else:
            inlaw = ""

        if sib_type == self.NORM_SIB:
            if not inlaw:
                if gender_b == Person.MALE:
                    rel_str = "le frère (germain)"
                elif gender_b == Person.FEMALE:
                    rel_str = "la sœur (germaine)"
                else:
                    rel_str = "le frère ou la sœur germain(e)"
            else:
                if gender_b == Person.MALE:
                    rel_str = "le beau-frère"
                elif gender_b == Person.FEMALE:
                    rel_str = "la belle-sœur"
                else:
                    rel_str = "le beau-frère ou la belle-sœur"
        elif sib_type == self.UNKNOWN_SIB:
            if not inlaw:
                if gender_b == Person.MALE:
                    rel_str = "le frère"
                elif gender_b == Person.FEMALE:
                    rel_str = "la sœur"
                else:
                    rel_str = "le frère ou la sœur"
            else:
                if gender_b == Person.MALE:
                    rel_str = "le beau-frère"
                elif gender_b == Person.FEMALE:
                    rel_str = "la belle-sœur"
                else:
                    rel_str = "le beau-frère ou la belle-sœur"
        elif sib_type == self.HALF_SIB_MOTHER:
            # for descendants the "half" logic is reversed !

            if gender_b == Person.MALE:
                rel_str = "le demi-frère consanguin"
            elif gender_b == Person.FEMALE:
                rel_str = "la demi-sœur consanguine"
            else:
                rel_str = "le demi-frère ou la demi-sœur consanguin(e)"
        elif sib_type == self.HALF_SIB_FATHER:
            # for descendants the "half" logic is reversed !

            if gender_b == Person.MALE:
                rel_str = "le demi-frère utérin"
            elif gender_b == Person.FEMALE:
                rel_str = "la demi-sœur utérine"
            else:
                rel_str = "le demi-frère ou la demi-sœur utérin(e)"
        elif sib_type == self.STEP_SIB:
            if gender_b == Person.MALE:
                rel_str = "le demi-frère"
            elif gender_b == Person.FEMALE:
                rel_str = "la demi-sœur"
            else:
                rel_str = "le demi-frère ou la demi-sœur"
        return rel_str


if __name__ == "__main__":
    # Test function. Call it as follows from the command line (so as to find
    #        imported modules):
    #    export PYTHONPATH=/path/to/gramps/src
    # python src/plugins/rel/rel_fr.py
    # (Above not needed here)

    """TRANSLATORS, copy this if statement at the bottom of your
    rel_xx.py module, and test your work with:
    python src/plugins/rel/rel_xx.py
    """
    from gramps.gen.relationship import test

    RC = RelationshipCalculator()
    test(RC, True)



#! /usr/bin/env python3
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2012       Benny Malengier
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
#

"""
This is a stub to start Gramps. It is provided for the sole reason of being
able to run gramps from the source directory without setting PYTHONPATH

From this position, import gramps works great
"""
import gramps.grampsapp as app

app.main()



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2007  Donald N. Allingham
# Copyright (C) 2009       Gary Burton
# Copyright (C) 2010       Nick Hall
# Copyright (C) 2011       Tim G L Lyons
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

"""
The EditAddress module provides the EditAddress class. This provides a
mechanism for the user to edit address information.
"""

# -------------------------------------------------------------------------
#
# Python modules
#
# -------------------------------------------------------------------------

# -------------------------------------------------------------------------
#
# GTK/Gnome modules
#
# -------------------------------------------------------------------------
import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

# -------------------------------------------------------------------------
#
# gramps modules
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.sgettext
from .editsecondary import EditSecondary
from gramps.gen.lib import NoteType
from ..glade import Glade
from .displaytabs import CitationEmbedList, NoteTab
from ..widgets import MonitoredDate, MonitoredEntry, PrivacyButton
from gramps.gen.const import URL_MANUAL_SECT3

# -------------------------------------------------------------------------
#
# Constants
#
# -------------------------------------------------------------------------

WIKI_HELP_PAGE = URL_MANUAL_SECT3
WIKI_HELP_SEC = _("Address_Editor_dialog", "manual")


# -------------------------------------------------------------------------
#
# EditAddress class
#
# -------------------------------------------------------------------------
class EditAddress(EditSecondary):
    """
    Displays a dialog that allows the user to edit an address.
    """

    def __init__(self, dbstate, uistate, track, addr, callback):
        """
        Displays the dialog box.

        parent - The class that called the Address editor.
        addr - The address that is to be edited
        """
        EditSecondary.__init__(self, dbstate, uistate, track, addr, callback)

    def _local_init(self):
        self.top = Glade()
        self.set_window(
            self.top.toplevel, self.top.get_object("title"), _("Address Editor")
        )
        self.setup_configs("interface.address", 650, 450)

    def _setup_fields(self):
        self.addr_start = MonitoredDate(
            self.top.get_object("date_entry"),
            self.top.get_object("date_stat"),
            self.obj.get_date_object(),
            self.uistate,
            self.track,
            self.db.readonly,
        )

        self.street = MonitoredEntry(
            self.top.get_object("street"),
            self.obj.set_street,
            self.obj.get_street,
            self.db.readonly,
        )

        self.city = MonitoredEntry(
            self.top.get_object("city"),
            self.obj.set_city,
            self.obj.get_city,
            self.db.readonly,
        )

        self.locality = MonitoredEntry(
            self.top.get_object("locality"),
            self.obj.set_locality,
            self.obj.get_locality,
            self.db.readonly,
        )

        self.state = MonitoredEntry(
            self.top.get_object("state"),
            self.obj.set_state,
            self.obj.get_state,
            self.db.readonly,
        )

        self.country = MonitoredEntry(
            self.top.get_object("country"),
            self.obj.set_country,
            self.obj.get_country,
            self.db.readonly,
        )

        self.postal = MonitoredEntry(
            self.top.get_object("postal"),
            self.obj.set_postal_code,
            self.obj.get_postal_code,
            self.db.readonly,
        )

        self.phone = MonitoredEntry(
            self.top.get_object("phone"),
            self.obj.set_phone,
            self.obj.get_phone,
            self.db.readonly,
        )

        self.priv = PrivacyButton(
            self.top.get_object("private"), self.obj, self.db.readonly
        )

    def _connect_signals(self):
        self.define_cancel_button(self.top.get_object("cancel"))
        self.define_ok_button(self.top.get_object("ok"), self.save)
        self.define_help_button(
            self.top.get_object("help"), WIKI_HELP_PAGE, WIKI_HELP_SEC
        )

    def _create_tabbed_pages(self):
        """
        Create the notebook tabs and inserts them into the main
        window.
        """

        notebook = Gtk.Notebook()

        self.srcref_list = CitationEmbedList(
            self.dbstate,
            self.uistate,
            self.track,
            self.obj.get_citation_list(),
            "address_editor_citations",
        )
        self._add_tab(notebook, self.srcref_list)
        self.track_ref_for_deletion("srcref_list")

        self.note_tab = NoteTab(
            self.dbstate,
            self.uistate,
            self.track,
            self.obj.get_note_list(),
            "address_editor_notes",
            notetype=NoteType.ADDRESS,
        )
        self._add_tab(notebook, self.note_tab)
        self.track_ref_for_deletion("note_tab")

        self._setup_notebook_tabs(notebook)
        notebook.show_all()
        self.top.get_object("vbox").pack_start(notebook, True, True, 0)

    def build_menu_names(self, obj):
        return (_("Address"), _("Address Editor"))

    def save(self, *obj):
        """
        Called when the OK button is pressed. Gets data from the
        form and updates the Address data structure.
        """
        if self.callback:
            self.callback(self.obj)
        self.close()



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
# Copyright (C) 2009       B. Malengier
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# python
#
# -------------------------------------------------------------------------
from gi.repository import GObject
from gi.repository import GLib

# -------------------------------------------------------------------------
#
# Gramps classes
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext
from gramps.gen.lib import EventRoleType, EventType
from gramps.gen.display.name import displayer as name_displayer
from .eventembedlist import EventEmbedList
from .eventrefmodel import EventRefModel
from gramps.gen.errors import WindowActiveError


# -------------------------------------------------------------------------
#
# PersonEventEmbedList
#
# -------------------------------------------------------------------------
class PersonEventEmbedList(EventEmbedList):
    _WORKNAME = _("Personal")
    # _FAMNAME = _("With %(namepartner)s (%(famid)s)")
    _FAMNAME = _("Family")
    _UNKNOWNNAME = _("<Unknown>")

    _MSG = {
        "add": _("Add a new personal event"),
        "del": _("Remove the selected personal event"),
        "edit": _("Edit the selected personal event or edit family"),
        "share": _("Share an existing event"),
        "up": _("Move the selected event upwards or change family order"),
        "down": _("Move the selected event downwards or change family order"),
    }

    def __init__(
        self,
        dbstate,
        uistate,
        track,
        obj,
        config_key,
        build_model=EventRefModel,
        **kwargs
    ):
        self.dbstate = dbstate
        EventEmbedList.__init__(
            self, dbstate, uistate, track, obj, config_key, build_model, **kwargs
        )

    def get_data(self):
        if not self._data or self.changed:
            self._data = [self.obj.get_event_ref_list()]
            self._groups = [(self.obj.get_handle(), self._WORKNAME, "")]
            # own family events
            family_handle_list = self.obj.get_family_handle_list()
            if family_handle_list:
                for family_handle in family_handle_list:
                    family = self.dbstate.db.get_family_from_handle(family_handle)
                    if family is None:
                        continue
                    father_handle = family.get_father_handle()
                    mother_handle = family.get_mother_handle()
                    if self.obj.get_handle() == father_handle:
                        handlepartner = mother_handle
                    else:
                        handlepartner = father_handle
                    if handlepartner:
                        partner = self.dbstate.db.get_person_from_handle(handlepartner)
                        groupname = name_displayer.display(partner)
                    else:
                        groupname = self._UNKNOWNNAME
                    self._data.append(family.get_event_ref_list())
                    self._groups.append((family_handle, self._FAMNAME, groupname))
            # we register all events that need to be tracked
            for group in self._data:
                self.callman.register_handles({"event": [eref.ref for eref in group]})
            self.changed = False

        return self._data

    def default_role(self):
        return EventRoleType(EventRoleType.PRIMARY)

    def default_types(self):
        return [
            EventType(EventType.BIRTH),
            EventType(EventType.DEATH),
            EventType(EventType.BURIAL),
        ]

    def get_ref_editor(self):
        from .. import EditEventRef

        return EditEventRef

    def editnotworkgroup(self, key):
        """
        Edit non native event in own editor
        """
        family = self.dbstate.db.get_family_from_handle(key)
        try:
            from .. import EditFamily

            EditFamily(self.dbstate, self.uistate, [], family)
        except WindowActiveError:
            pass

    def _non_native_change(self):
        """
        handle change request of non native data
        """
        from ...dialog import WarningDialog

        WarningDialog(
            _("Cannot change Family"),
            _("You cannot change Family events in the Person Editor"),
            parent=self.uistate.window,
        )

    def _move_up_group(self, groupindex):
        """
        move up pressed on the group
        If on a family group, allow to reorder the order of families
        as connected to this person.
        """
        ref = self.get_selected()
        if not ref or ref[1]:
            return
        if ref[0] == 0:
            # the workgroup: person events, cannot move up
            return
        if ref[0] == 1:
            # cannot move up, already first family
            return
        # change family list and rebuild the tabpage
        index = ref[0] - 1
        flist = self.obj.get_family_handle_list()
        handle = flist.pop(index)
        flist.insert(index - 1, handle)
        self.changed = True
        self.rebuild()
        # select the row
        # New index is index-1 but for path, add another 1 for person events.
        path = (index,)
        self.tree.get_selection().select_path(path)
        GLib.idle_add(self.tree.scroll_to_cell, path)

    def _move_down_group(self, groupindex):
        """
        move down pressed on the group
        If on a family group, allow to reorder the order of families
        as connected to this person.
        """
        ref = self.get_selected()
        if not ref or ref[1]:
            return
        if ref[0] == 0:
            # person events, cannot move down
            return
        if ref[0] == len(self._groups) - 1:
            # cannot move down, already last family
            return
        # change family list and rebuild the tabpage
        index = ref[0] - 1
        flist = self.obj.get_family_handle_list()
        handle = flist.pop(index)
        flist.insert(index + 1, handle)
        self.changed = True
        self.rebuild()
        # select the row
        # New index is index+1 but for path, add another 1 for person events.
        path = (index + 2,)
        self.tree.get_selection().select_path(path)
        GLib.idle_add(self.tree.scroll_to_cell, path)



#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# -------------------------------------------------------------------------
#
# Python classes
#
# -------------------------------------------------------------------------
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.gettext
from gi.repository import GObject, GLib

# -------------------------------------------------------------------------
#
# Gramps classes
#
# -------------------------------------------------------------------------
from gramps.gen.lib import Attribute
from gramps.gen.errors import WindowActiveError
from ...ddtargets import DdTargets
from .attrmodel import AttrModel
from .embeddedlist import EmbeddedList, TEXT_COL, MARKUP_COL, ICON_COL


# -------------------------------------------------------------------------
#
#
#
# -------------------------------------------------------------------------
class AttrEmbedList(EmbeddedList):
    _HANDLE_COL = 4
    _DND_TYPE = DdTargets.ATTRIBUTE

    _MSG = {
        "add": _("Create and add a new attribute"),
        "del": _("Remove the existing attribute"),
        "edit": _("Edit the selected attribute"),
        "up": _("Move the selected attribute upwards"),
        "down": _("Move the selected attribute downwards"),
    }

    # index = column in model. Value =
    #  (name, sortcol in model, width, markup/text, weigth_col
    _column_names = [
        (_("Type"), 0, 250, TEXT_COL, -1, None),
        (_("Value"), 1, 200, TEXT_COL, -1, None),
        (_("Source"), 2, 30, ICON_COL, -1, "gramps-source"),
        (_("Private"), 3, 30, ICON_COL, -1, "gramps-lock"),
    ]

    def __init__(self, dbstate, uistate, track, data, config_key):
        """
        Initialize the displaytab. The dbstate and uistate is needed
        track is the list of parent windows
        data is an attribute_list (as obtained by AttributeBase) to display and
            edit
        """
        self.data = data
        EmbeddedList.__init__(
            self,
            dbstate,
            uistate,
            track,
            _("_Attributes"),
            AttrModel,
            config_key,
            move_buttons=True,
        )

    def get_editor(self):
        from .. import EditAttribute

        return EditAttribute

    def get_user_values(self):
        return self.dbstate.db.get_person_attribute_types()

    def get_icon_name(self):
        return "gramps-attribute"

    def get_data(self):
        return self.data

    def column_order(self):
        return ((1, 2), (1, 3), (1, 0), (1, 1))

    def add_button_clicked(self, obj):
        pname = ""
        attr = Attribute()
        try:
            self.get_editor()(
                self.dbstate,
                self.uistate,
                self.track,
                attr,
                pname,
                self.get_user_values(),
                self.add_callback,
            )
        except WindowActiveError:
            pass

    def add_callback(self, name):
        data = self.get_data()
        data.append(name)
        self.changed = True
        self.rebuild()
        GLib.idle_add(self.tree.scroll_to_cell, len(data) - 1)

    def edit_button_clicked(self, obj):
        attr = self.get_selected()
        if attr:
            pname = ""
            try:
                self.get_editor()(
                    self.dbstate,
                    self.uistate,
                    self.track,
                    attr,
                    pname,
                    self.get_user_values(),
                    self.edit_callback,
                )
            except WindowActiveError:
                pass

    def edit_callback(self, name):
        self.changed = True
        self.rebuild()



# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2007-2009  Douglas S. Blank <doug.blank@gmail.com>
# Copyright (C) 2020       Dave Scheipers
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# ------------------------------------------------------------------------
#
# Gtk modules
#
# ------------------------------------------------------------------------
from gi.repository import Gtk

# ------------------------------------------------------------------------
#
# Gramps modules
#
# ------------------------------------------------------------------------
from gramps.gen.const import URL_WIKISTRING, URL_MANUAL_PAGE, URL_HOMEPAGE
from gramps.gen.const import WIKI_EXTRAPLUGINS
from gramps.gui.display import EXTENSION
from gramps.gen.plug import Gramplet
from gramps.gui.widgets.styledtexteditor import StyledTextEditor
from gramps.gen.lib import StyledText, StyledTextTag, StyledTextTagType
from gramps.gen.const import GRAMPS_LOCALE as glocale

_ = glocale.translation.sgettext


# ------------------------------------------------------------------------
#
# Functions
#
# ------------------------------------------------------------------------
def st(text):
    """Return text as styled text"""
    return StyledText(text)


def boldst(text):
    """Return text as bold styled text"""
    tags = [StyledTextTag(StyledTextTagType.BOLD, True, [(0, len(text))])]
    return StyledText(text, tags)


def linkst(text, url):
    """Return text as link styled text"""
    tags = [StyledTextTag(StyledTextTagType.LINK, url, [(0, len(text))])]
    return StyledText("  • ") + StyledText(text, tags)


def wiki(page, manual=False):
    """
    Build a link to a wiki page.
    """
    url = URL_WIKISTRING
    if manual:
        url += URL_MANUAL_PAGE
    url += page + EXTENSION
    return url


# ------------------------------------------------------------------------
#
# Gramplet class
#
# ------------------------------------------------------------------------


class WelcomeGramplet(Gramplet):
    """
    Displays a welcome note to the user.
    """

    def init(self):
        self.gui.WIDGET = self.build_gui()
        self.gui.get_container_widget().remove(self.gui.textview)
        self.gui.get_container_widget().add(self.gui.WIDGET)
        self.gui.WIDGET.show()

    def build_gui(self):
        """
        Build the GUI interface.
        """
        top = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)

        scrolledwindow = Gtk.ScrolledWindow()
        scrolledwindow.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
        self.texteditor = StyledTextEditor()
        self.texteditor.set_editable(False)
        self.texteditor.set_wrap_mode(Gtk.WrapMode.WORD)
        self.texteditor.set_left_margin(6)
        self.texteditor.set_right_margin(6)
        scrolledwindow.add(self.texteditor)

        self.display_text()

        top.pack_start(scrolledwindow, True, True, 0)
        top.show_all()
        return top

    def display_text(self):
        """
        Display the welcome text.
        """
        welcome = boldst(_("Intro")) + "\n\n"
        welcome += _(
            "Gramps is a software package designed for genealogical research."
            " Although similar to other genealogical programs, Gramps offers"
            " some unique and powerful features.\n\n"
        )
        welcome += linkst(_("Home Page"), URL_HOMEPAGE) + "\n\n"

        welcome += (
            boldst(_("Who makes Gramps?"))
            + "\n\n"
            + _(
                "Gramps is created by genealogists for genealogists, organized in"
                " the Gramps Project. "
                "Gramps is an Open Source Software package, which means you are"
                " free to make copies and distribute it to anyone you like. It's"
                " developed and maintained by a worldwide team of volunteers whose"
                " goal is to make Gramps powerful, yet easy to use.\n\n"
            )
        )
        welcome += _(
            "There is an active community of users available on the mailing"
            " lists and Discourse forum to share ideas and techniques.\n\n"
        )
        welcome += linkst(_("Gramps online manual"), wiki("", manual=True)) + "\n\n"
        welcome += (
            linkst(
                _("Ask questions on gramps-users mailing list"),
                "%(gramps_home_url)scontact/" % {"gramps_home_url": URL_HOMEPAGE},
            )
            + "\n\n"
        )
        welcome += (
            linkst(_("Gramps Discourse Forum"), "https://gramps.discourse.group/")
            + "\n\n"
        )
        welcome += (
            boldst(_("Getting Started"))
            + "\n\n"
            + _(
                "The first time Gramps is started all of the Views are blank. There"
                " are very few menu options. A Family Tree is needed for any activity"
                " to happen.\n\n"
                "To create a new Family Tree (sometimes called 'database') select"
                ' "Family Trees" from the menu, pick "Manage Family Trees", press'
                ' "New" and name your Family Tree. "Load Family Tree" to make the'
                " tree active and ready to accept data by entering your first"
                " family, or importing a family tree. For more details, please"
                " read the information at the links below.\n\n"
            )
        )
        welcome += (
            linkst(_("Start with Genealogy and Gramps"), wiki("Start_with_Genealogy"))
            + "\n\n"
        )
        welcome += (
            boldst(_("Enter your first Family"))
            + "\n\n"
            + _(
                "You will now want to start entering your first Family and that"
                " starts with the first Person.\n\n"
                'Switch to the "People" view and from the menu clicking "Add" and then'
                ' clicking "Person" (or using the [+] icon) will bring up the window'
                " to enter a person. Entering the basic information and saving the"
                " record gives you a starting point. Select this Person's record and"
                ' now switch to the "Relationships" view.\n\n'
                "With this first person, all of the menu options and icon functions"
                " have become available. Spend some time moving your mouse over the"
                " icons. As your cursor passes over an icon, a message will appear"
                " telling you the icon's function. The same is true for any of the"
                " edit windows. Moving the mouse cursor over an item will tell you"
                " what it will do.\n\n"
                "You can now create families by adding parents, a spouse and children."
                " Once started, you will be able to add Events to People and Families."
                " You can provide Sources and Citations to provide documentation for"
                " your entries.\n\n"
                "As you start using Gramps, you will find that information can be"
                " entered from all the various Views. There are multiple ways of"
                " doing most activities in Gramps. The flexibility allows you to"
                " choose which fits your work style.\n\n"
            )
        )
        welcome += (
            linkst(
                _("Entering and editing data (brief)"),
                wiki("_-_Entering_and_editing_data:_brief", manual=True),
            )
            + "\n\n"
        )
        welcome += (
            boldst(_("Importing a Family Tree"))
            + "\n\n"
            + _(
                "To import a Family Tree from another program first create"
                " a GEDCOM (or other data) file from the previous program.\n\n"
                'Once you have created a new Gramps database file, use the "Import"'
                ' option under the "Family Trees" menu to import the GEDCOM data.\n\n'
            )
        )
        welcome += (
            linkst(
                _("Import from another genealogy program"),
                wiki("Import_from_another_genealogy_program"),
            )
            + "\n\n"
        )
        welcome += (
            boldst(_("Dashboard View"))
            + "\n\n"
            + _(
                'You are currently reading from the "Dashboard" view, where you can'
                " add your own gramplets. You can also add gramplets to any view by"
                " adding a sidebar and/or bottombar, and right-clicking to the"
                " right of the tab.\n\n"
                "You can click the configuration icon in the toolbar to add"
                " additional columns, while right-click on the background allows to"
                " add gramplets. You can also drag the Properties button to"
                " reposition the gramplet on this page, and detach the gramplet to"
                " float above Gramps."
            )
        )
        welcome += "\n\n"
        welcome += (
            linkst(_("Gramps View Categories"), wiki("_-_Categories", manual=True))
            + "\n\n"
        )
        welcome += (
            boldst(_('Addons and "Gramplets"'))
            + "\n\n"
            + _(
                'There are many Addons or "Gramplets" that are available to assist you'
                " in data entry and visualizing your family tree. Many of these tools"
                " are already available to you. Many more are available to download"
                " and install.\n\n"
            )
        )
        welcome += linkst(_('Addons and "Gramplets"'), wiki(WIKI_EXTRAPLUGINS)) + "\n\n"
        welcome += (
            boldst(_("Example Database"))
            + "\n\n"
            + _(
                "Want to see Gramps in use. Create and Import the Example database.\n\n"
                "Create a new Family Tree as described above. Suggest that you name"
                " the Family Tree “EXAMPLE”.\n\n"
                "Import the Gramps file example.gramps.\n\n"
                "Follow the instructions for the location of the file stored with"
                " the Gramps program.\n\n"
            )
        )
        welcome += linkst("Example.gramps", wiki("Example.gramps")) + "\n\n"

        self.texteditor.set_text(welcome)

    def get_has_data(self, obj):
        """
        Return True if the gramplet has data, else return False.
        """
        return True
